/** *************************************************************************************
 *
 * PROGRAM:
 *
 * 	microA: implements simulations to perform matting between individuals given some traits x and y depending on a mating preference function.
 *			The matings will occur in different microareas.
 *          However the measurement of sexual isolation will be performed ignoring the microarea effect.
 *          This will allow us to measure the microarea impact when measuring sexual isolation.
 *
 *	Preference Functions:
 *
 *		Bolnick & Doebeli 2003, Doebeli 2005
 *
 *			c>0.5  e^–(0.5Q^2D^2)/V
 *			c<0.5   e^–(0.5Q^2(Dmax-D)^2)/V
 *
 *		Gavrilets et al 2007
 *          K0 (1000-50000) Nini = 50 (Sadedin 2009)
 *			c>0.5   e^–(0.5QD^2)/V
 *			c<0.5   e^-0.5QD/Ve-Qf(x-1)/V e-Qx(f-1)/V e-0.5Q/V
 *
 *		C-R & RA 2014
 *			c>0.5   FND
 *			c<0.5   FND
 *
 *		TP&G 2013
 *			c>0.5   e^–(0.5QD^2)/V (=p1)
 *			c<0.5   2 - e^–(0.5QD^2)/V (=2-p1)
 *
 *      Where m = 2c -1; c (choice): > 0.5 associative positive; <0.5 associative negative; 0.5 random
 *      Q = m^2; D: |x-y|; x: female ecological trait; y: male ecological trait;
 *      V = s^2;
 *      s: scaling parameter that determines the slope of the mate-choice function {Bolnick, 2003 #5773};
 *
 *  ALGORITHM
 *
 *	1. Se decide el valor de media (Mp) y varianza (Vp) del carácter en la metapoblación. Numero de microareas = 50.
 *	2. Se muestrean 50 (numero de microareas) valores de N(Mp,Vp) que serán los valores medios del carácter en las diferentes microareas (M1, M2.. M50)
 *	3. Se extrae una pareja de cada microarea utilizando la Media de su microarea y la varianza poblacional, y por supuesto utilizando la funcion de preferencia
 *	   con el nivel adecuado de s2 y C, con el fin de obtener un determinado valor de apareamiento asociativo, ya conocido dadas nuestras simulaciones.
 *		Se extrae una pareja se calcula su probabilidad y se ve si aparea, si no se repite el proceso hasta que se aparee o hasta maxiter que entonces se aparea al azar
 *	4. Se calcula para el comjunto de microareas el valor de correlacion de tdas las copulas generadas, y se obtiene r (variable RESPUESTA principal)
 *	5. Se genera la distribucion de la variable respuesta utilizando las 1000 réplicas...


 *	Arguments: Mp, Vp, # microareas, s^2, C
 *
 * 	http:
 *
 * AUTHOR:
 *
 * 	Antonio Carvajal Rodriguez (acraaj@uvigo.es) October 2013
 *
 * 	Area de Genetica. Universidad de Vigo.
 *
 * 	Group of Genetics of Populations and Cytogenetics (XB2)
 *
 * *************************************************************************************
 *
 * DISCLAIMER:
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version. This program is
 * distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details. You should have received a copy of the GNU
 * General Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. See the file
 * "gpl.html" under the "license" directory.
 *
 * *************************************************************************************
 *
 * VERSION HISTORY:
 *
 * ************************************************************************************* */


# include <iomanip>

# include "microA.h"

#ifndef macintosh
	#include <sys/types.h>
	#include <sys/stat.h>
#endif


using namespace std;


class Experiment{
        static const int missingvalue=-99;


		struct casos {
			int replicas;
			int Kmax;
			double minpheno;
			double maxpheno;
			double s;
			double inichoice;
			double endchoice;
			double choiceinc;
		} caso;
        bool errflag,unbiasedcorrel;
        int funflag;
	public:

            Experiment(){
                errflag=false;
                unbiasedcorrel=true;
            }


// mating functions, all must be of the kind double fun(const double& V,const double& choice,const double& Zm,const double& Zf)
// in order to be passed as arguments as a pointer to function in the computedistrib function they also must be static
// Dieckmann &Doebeli 1999. The original function uses choice^4 that in our notation means exponent 2 so choice^2 (=Q) is the base
    static double DD99(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = 200*m;
            double D = fabs(Zm-Zf);
            D*=D;
            double K = Q*(D);
            if(choice>0) // positive association
                p=exp(-K);
            else //negative association
                p=1.0-exp(-0.5*m*D);
        }
        return p;
	}

    static double Gauss(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = (choice*choice);
            double Q = m/(2*V);
            double D = fabs(Zm-Zf);
            D*=D;
            double K = Q*(D);
            p=exp(-K);
            if(choice<0) //negative association
                p=1.0-p;
        }
        return p;
	}

// Bolnick &Doebeli 2003 expoenet must be 2
    static double BD03(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = 0.5*m;
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = Q*(D*D)/V;
                p=exp(-K);
            }
            else{ //negative association
                double K = pow((maxrange-D),2.0);
                K*=Q;
                K/=V;

                p=exp(-K);

            }
        }
        return p;
	}


// Carvajal-Rodriguez 2013 folded normal for the variable D = |Zf-Zm| and sigma. Sigma must be computed out of the function
    static double FND(const double& maxrange,const double& sigma,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;

        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{

            double Q = sqrt(2)/(sigma*sqrt(trigPI));
            double D = fabs(Zm-Zf);
            double V = sigma*sigma;


            if(choice>0){ // positive association

                double K = (D*D)/(2*V);

                p=min(Q*exp(-K),1.0);
            }
            else{ //negative association

                double K = pow((maxrange-D),2.0);

                K/=(2*V);

                p=min(Q*exp(-K),1.0); // version s/C
            }
        }
		p=max(p,0.0);
        return p;
	}

// Carvajal-Rodriguez 2012 folded normal for the 2D variable D = |Zf-Zm| V=s/2C
    static double FN2D_2c(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = fabs(choice)/(sqrt(V*trigPI));
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                //double K = 4*m*(D*D)/V; // version s/4C
                double K = m*(D*D)/V; //version S/2C
                // p=min(4*Q*exp(-K),1.0); // version s/4C
                p=min(2*Q*exp(-K),1.0);
            }
            else{ //negative association
                double K = pow((maxrange-D),2.0);
                //K*=(4*m); // version s/4C
                K*=(m); //version S/2C
                K/=V;
                // p=min(4*Q*exp(-K),1.0); // version s/4C
                p=min(2*Q*exp(-K),1.0); // version s/4C
            }
        }
		p=max(p,0.0);
        return p;
	}

// Carvajal-Rodriguez 2012 folded normal for the 2D variable D = |Zf-Zm| V=s/4C
    static double FN2D_4c(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = fabs(choice)/(sqrt(V*trigPI));
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = 4*m*(D*D)/V; // version s/4C
                p=min(4*Q*exp(-K),1.0); // version s/4C
            }
            else{ //negative association
                double K = pow((maxrange-D),2.0);
                K*=(4*m); // version s/4C
                K/=V;
                p=min(4*Q*exp(-K),1.0); // version s/4C

            }
        }
		p=max(p,0.0);
        return p;
	}

// Gavrilets 07. Exponent must be 1 for the exact Gavrilets formulae
	static double G07(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = 0.5*m;
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = Q*(D*D)/V;
                p=exp(-K);
            }
            else{ //negative association
                double K = pow((Zm+Zf-1.0),2.0);
                K*=Q;
                K/=V;
                p=exp(-K);
            }
        }
        return p;
	}

	// Modified from Gavrilets 07. It uses maxrange=Zm+Zf-(max(Zm,Zf)+min(Zm,Zf))/2 instead Zm+Zf-1.
	static double G07B(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = 0.5*m;
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = Q*(D*D)/V;
                p=exp(-K);
            }
            else{ //negative association
                double K = pow((Zm+Zf-maxrange),2.0);
                K*=Q;
                K/=V;
                p=exp(-K);
            }
        }
        return p;
	}


	//Debarre12 exponent must be 1
	static double D12(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow((choice*choice),exponent);
            double Q = m;
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = (D*D)/(2*V);
                p=exp(-K);
                p=1.0-p;
                p*=Q;
                p=1.0-p;
            }
            else{ //negative association
                double K = (D*D)/(2*V);
                p=exp(-K);
                p*=Q;
                p=1.0-p;
            }
        }
        return p;
	}

    //Carvajal-Rodriguez&Rolan-Alvarez13. exponent must in principle be 1 but we can try 2 or more..
	static double CR13(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf){
        double p;
        if(choice==0) return 1.0; // random mating so this Zm and Zf will mate
        else{
            double m = pow(fabs(choice),exponent); // use with exponent <=1 (D12 uses exponent 2)
            double Q = m*m;
            double D = fabs(Zm-Zf);

            if(choice>0){ // positive association
                double K = pow(D,2.0)/(2*V);
                p=exp(-Q*K);
                p=1.0-p;
                p*=m;
                p=1.0-p;
            }
            else{ //negative association
                double K = pow(D,2.0)/(2*V);
                p=exp(-Q*K);
                p*=m;
                p=1.0-p;
            }

        }
        return p;
	}
    // choice is m = 2c-1 Zm is the male trait, Zf is the female trait, V is the Va or s the scaling parameter
    //The argument function fun is any mating function having the given arguments and returning a double
	vector<double> computedistrib(const double& maxrange,const double& V,const double& choice,const double& exponent,const vector<double>& choosersexphenotypes,const vector<double>& chosedphenotypes, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){
        vector<double>temp(choosersexphenotypes.size());
        double acum=0.0;
        double fem,male;
        for(int m=0;m<choosersexphenotypes.size();m++){
            //fem=femalephenotypes[uniform()*femalephenotypes.size()];
            male = choosersexphenotypes[m];
            for(int f=0;f<chosedphenotypes.size();f++){
                fem = chosedphenotypes[f];
                acum+=fun(maxrange,V,choice,exponent,male,fem);
            }
            temp[m]=acum;

            acum=0.0;
        }


        return temp;

	}
// the same as previous but implemented as procedure not function to avoid the local temporal object construction destruction

	void computedistrib(vector<double> & distribution, const double& maxrange,const double& V,const double& choice,const double& exponent,const vector<double>& choosersexphenotypes,const vector<double>& chosedphenotypes, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){

        if(distribution.size()!= choosersexphenotypes.size()){
            string erm = "Proiblem with distribution size in computedistrib = "+NtoS(distribution.size())+"\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);
        }
        double acum=0.0;
        double fem,male;
        for(int m=0;m<choosersexphenotypes.size();m++){
            //fem=femalephenotypes[uniform()*femalephenotypes.size()];
            male = choosersexphenotypes[m];
            for(int f=0;f<chosedphenotypes.size();f++){
                fem = chosedphenotypes[f];
                acum+=fun(maxrange,V,choice,exponent,male,fem);
            }
            distribution[m]=acum;

            acum=0.0;
        }

	}


double computecorrel(const vector<double>& males,const vector<double>& females,const int ini,const int end,double& meanx,double& meany, double& sdx, double&sdy){

		vector<double>x;
		vector<double>y;

		double avex=0.0,avey=0.0,varx=0.0,vary=0.0;

        if(ini<0){
                string erm = "Error in initial position of range = "+NtoS(ini)+"\n";
                cout<<erm<<endl;
                CppTools::texterr("Spectest",erm,true);
        }

        if(end>males.size()){
                string erm = "Error in ending position of range = "+NtoS(end)+"\n";
                cout<<erm<<endl;
                CppTools::texterr("Spectest",erm,true);
        }

        int tam = end-ini;
//cout<<males.size()<<endl;
//cout<<females.size()<<endl;
        for(int m=ini;m<end;m++){
//cout<<m<<"\t"<<females[m]<<"\t"<<males[m]<<endl;
            x.push_back(males[m]);
            y.push_back(females[m]);
            avex+=males[m];
            avey+=females[m];
            varx+=(males[m]*males[m]);
            vary+=(females[m]*females[m]);

        } // close for

        if(x.size()!=tam){
            string erm = "Proiblem with x size in computecorrel = "+NtoS(x.size())+"\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);

        }
        if(y.size()!=tam){
            string erm = "Proiblem with x size in computecorrel = "+NtoS(y.size())+"\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);

        }
        avex/=tam;
        avey/=tam;

        varx/=tam;
        vary/=tam;
        varx-=(avex*avex);
        vary-=(avey*avey);

        if(varx<0)
            if(fabs(varx<1e-6)) varx=0.0;
            else{
                string erm = "Negative variance in x when computing correlation = "+NtoS(varx)+"\n";
                cout<<erm<<endl;
                CppTools::texterr("Spectest",erm,true);
            }
        if(vary<0)
            if(fabs(vary<1e-6)) vary=0.0;
            else{
                string erm = "Negative variance in y when computing correlation = "+NtoS(vary)+"\n";
                cout<<erm<<endl;
                CppTools::texterr("Spectest",erm,true);
            }

        // deviate values from mean to compute correlation using correlxy

		for(int m=0;m<x.size();m++){
            x[m]-=avex;
            y[m]-=avey;

		} // close for

        meanx=avex;
        meany=avey;
        sdx=sqrt(varx);
        sdy=sqrt(vary);

        double correl=0;
        correl = CppTools::correlxy(x,y);

        x.clear();
        y.clear();
        vector<double>().swap(x);
        vector<double>().swap(y);

		return correl;

	}


    // choice is m = 2c-1 Zm is the male trait, Zf is the female trait, V is the Va or s the scaling parameter
    //The argument function fun is any mating function having the given arguments and returning a double
	// this function will perform mating and compute the correlation between mating phenotypes
	//probabilites are NOT normalized
	double mating(struct casos& caso,const int run,string& infloop,const int fid, const double& maxrange,const double& V,const double& choice,const double& exponent,const vector<double>& choosersexphenotypes,const vector<double>& chosedphenotypes, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){

		vector<double>x(choosersexphenotypes.size());
		vector<double>y(chosedphenotypes.size());
		double distancia;
		double prob=0.0;
		double fem,male,comodin,desperateprob=-1;
		double avex=0.0,avey=0.0;

		//maxiterations = 4*choosersexphenotypes.size();
		//ACR0513
		//if(maxiterations<100) maxiterations=100;
		int C;
        funflag=fid;

        x=choosersexphenotypes;

        if(choice==0){
            for(int m=0;m<choosersexphenotypes.size();m++){

                male = choosersexphenotypes[m];
                avex+=male;
                fem = chosedphenotypes[uniform()*chosedphenotypes.size()];
                // if choice is 0 do not need to check prob which is always 1
                //prob=fun(maxrange,V,choice,exponent,male,fem);

                y[m]=fem;
                avey+=fem;


            } // close for

        }
		else
		for(int m=0;m<choosersexphenotypes.size();m++){
			if(choice>0) distancia=maxrange;
            else distancia=0;
			C=0;
			male = choosersexphenotypes[m];
            avex+=male;
			do{
				fem = chosedphenotypes[uniform()*chosedphenotypes.size()];
				prob=fun(maxrange,V,choice,exponent,male,fem);
//cout<<fem<<"\t"<<prob<<endl;
				if(choice>0){
                    if(distancia>fabs(male-fem)){
                        distancia=fabs(male-fem);
                        comodin=fem;
                        desperateprob=prob;
                    }
				}
				else{
                    if(distancia<fabs(male-fem)){
                        distancia=fabs(male-fem);
                        comodin=fem;
                        desperateprob=prob;
                    }
				}
				C++;
			}while( (uniform()>prob) && C<maxiterations);
			if(C>=maxiterations){
			    fem=comodin;
			    prob=desperateprob;
			    string function;
			    switch(fid){
			    case 0: function="DD99";break;
			    case 1: function="BD03a";break;
			    case 2: function="G07B";break;
			    case 3: function="Gauss";break;
			    case 4: function="D12";break;
			    case 5: function="FND";break;
			    default:function="Wrong function identifier";
			    }
				//string erm = "Caso Kmax= "+NtoS(caso.Kmax)+" Va= "+NtoS(caso.s)+ ". Run "+NtoS(run)+". Male number "+NtoS(m) + ". Infinite loop in mating. C = "+NtoS(C)+" in function "+function+" with choice "+NtoS(choice)+". Male was "+NtoS(male)+". Chosed female was "+NtoS(comodin)+". Mating prob was "+NtoS(prob)+"\n";
				//infloop+=erm;
				//cout<<erm<<endl;
				//CppTools::texterr("Spectest",erm,false);
			}

			y[m]=fem;
			avey+=fem;

		} // close for

        avex/=x.size();
        avey/=y.size();
        for(int m=0;m<choosersexphenotypes.size();m++){
            x[m]-=avex;
            y[m]-=avey;
        }

        double rxy=CppTools::correlxy(x,y,errflag,unbiasedcorrel);
        if(errflag) rxy=missingvalue; // if there was a problem with variance 0 in x or y

		return rxy;

	}
// Isolation . Compute the Levene isolation index

    double Isolation(const int fid,const double& maxrange, const double& rangeini,const double& rangeend,const double& V,const double& choice,const double& exponent,double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){


		double tot;
		double prob1,prob2,prob3,prob4;
		double isolation;

        prob1=fun(maxrange,V,choice,exponent,rangeini,rangeini);
        prob2=fun(maxrange,V,choice,exponent,rangeini,rangeend);
        prob3=fun(maxrange,V,choice,exponent,rangeend,rangeini);
        prob4=fun(maxrange,V,choice,exponent,rangeend,rangeend);
        tot= prob1+prob2+prob3+prob4;
        isolation=prob1+prob4-(prob2+prob3);
        if(tot>0)
            isolation/=tot;
        else{

            string erm="Isolation nan in fun ";
            switch(fid){
			    case 0: erm+="DD99_C=";break;
			    case 1: erm+="BD03a_C=";break;
			    case 2: erm+="G07B_C=";break;
			    case 3: erm+="Gauss_C=";break;
			    case 4: erm+="D12_C=";break;
			    case 5: erm+="FND_C=";break;
			    default:erm+="Wrong function identifier";
            }
            erm+= NtoS(choice);
            erm+=("_A="+NtoS(rangeini)+"_B="+NtoS(rangeend) +"_V="+NtoS(V)+"_tot="+NtoS(tot)+"_p1="+NtoS(prob1)+"_p2="+NtoS(prob2)+"_p3="+NtoS(prob3)+"_p4="+NtoS(prob4));
            //cout<<erm<<endl;
            //CppTools::texterr("Spectest",erm,true);

           isolation=missingvalue;
        }
        if(fabs(isolation)>1 && isolation!=missingvalue){

            string erm="Isolation>1 in fun ";
            switch(fid){
			    case 0: erm+="DD99_C=";break;
			    case 1: erm+="BD03a_C=";break;
			    case 2: erm+="G07B_C=";break;
			    case 3: erm+="Gauss_C=";break;
			    case 4: erm+="D12_C=";break;
			    case 5: erm+="FND_C=";break;
			    default:erm+="Wrong function identifier";
            }
           erm+= NtoS(choice);
            erm+=("_I="+NtoS(isolation)+ "_A="+NtoS(rangeini)+"_B="+NtoS(rangeend) +"_V="+NtoS(V)+"_tot="+NtoS(tot)+"_p1="+NtoS(prob1)+"_p2="+NtoS(prob2)+"_p3="+NtoS(prob3)+"_p4="+NtoS(prob4));
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);
        }
		return isolation;

	}


	void testfunction(const int fid,vector<double>& fvals, const double& maxrange,const double& V,const double& choice,const double& exponent,const vector<double>& choosersexphenotypes,const vector<double>& chosedphenotypes, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){
		vector<double>temp(choosersexphenotypes.size());

		double fem,male;

		for(int m=0;m<choosersexphenotypes.size();m++){

			male = choosersexphenotypes[m];
            fem = chosedphenotypes[m];
            temp[m]=fun(maxrange,V,choice,exponent,male,fem);


		} // close for

        fvals=temp;

	}

    // choice is m = 2c-1 Zm is the male trait, Zf is the female trait, V is the Va or s the scaling parameter
    //The argument function fun is any mating function having the given arguments and returning a double
	// this function will perform mating and compute the correlation between mating phenotypes
	//probabilites will be normalized with the sum for each male with every female
	// because the mating probability of each male is 1 the mating is performed using a roulette wheel over the females
	//PROBABILITES ARE NORMALIZED BY DISTRIB
	double mating(const int fid,const double& maxrange,const double& V,const double& choice,const double& exponent,const vector<double>& choosersexphenotypes,const vector<double>& chosedphenotypes,const vector<double>& distrib, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){

		vector<double>x(choosersexphenotypes.size());
		vector<double>y(chosedphenotypes.size());
		double prob=0.0;
		double fem,male;
		double distancia;
		double avex=0.0,avey=0.0;
		double checkprob=0;
		//maxiterations = 4*choosersexphenotypes.size();
		//ACR0513
		//if(maxiterations<100) maxiterations=100;
		int C;

        funflag=fid;
        x=choosersexphenotypes;
        if(choice==0){
            for(int m=0;m<choosersexphenotypes.size();m++){

                male = choosersexphenotypes[m];
                avex+=male;
                fem = chosedphenotypes[uniform()*chosedphenotypes.size()];
                prob=fun(maxrange,V,choice,exponent,male,fem);

                y[m]=fem;
                avey+=fem;


            } // close for

        }
		else
		for(int m=0;m<choosersexphenotypes.size();m++){


			checkprob=0;
			C=0;
			if(choice>0) distancia=1;
            else distancia=0;
			male = choosersexphenotypes[m];
            avex+=male;
			do{
				fem = chosedphenotypes[uniform()*chosedphenotypes.size()];
				prob=fun(maxrange,V,choice,exponent,male,fem);
				prob/=distrib[m];
                checkprob+=prob;

				C++;
			}while( (uniform()>checkprob) && C<maxiterations);
			if(C>=maxiterations){


				string function;
			    switch(fid){
			    case 0: function="DD99";break;
			    case 1: function="BD03a";break;
			    case 2: function="BD03b";break;
			    case 3: function="Gauss";break;
			    case 4: function="D12";break;
			    case 5: function="CR13";break;
			    default:function="Wrong function identifier";
			    }
				//string erm = "Infinite loop in mating. C = "+NtoS(C)+" in normalized function "+function+" with choice "+NtoS(choice)+". Male was "+NtoS(male)+". Chosed female was "+NtoS(fem)+". Mating prob was "+NtoS(prob);
				//cout<<erm<<endl;
				//CppTools::texterr("Spectest",erm,false);
			}
			y[m]=fem;
			avey+=fem;


		} // close for

        avex/=x.size();
        avey/=y.size();
        for(int m=0;m<choosersexphenotypes.size();m++){
            x[m]-=avex;
            y[m]-=avey;
//cout<<x[m]<<"\t"<<y[m]<<endl;
        }
        double rxy=CppTools::correlxy(x,y,errflag,unbiasedcorrel);
        if(errflag) rxy=missingvalue; // if there was a problem with variance 0 in x or y
		return rxy;

	}

	vector<double> generatePhenotypes(const int popsize, const double& mean, const double& sdev){
		vector<double> phenotypes(popsize);
			for(int i=0; i< popsize;i++)
				phenotypes[i]=normal(mean,sdev);
		return phenotypes;
	}

    void generatePhenotypes(vector<double>&phenotypes, const double& mean, const double& sdev){
		int popsize=phenotypes.size();
        for(int i=0; i< popsize;i++)
            phenotypes[i]=normal(mean,sdev);

	}
// This function first scale the minimum to be 0, then scale the maximum to be scalefactor. Then shift to scaleini. This scaling is not just a translation so do affect to the variance
// Return the maximum raw value
	double scalephenotypes(const double scaleini,const double scalefactor, vector<double>& malephenotypes,vector<double>& femphenotypes){

		vector<double> phenotypes;

		for(int i=0; i<malephenotypes.size();i++)
			phenotypes.push_back(malephenotypes[i]);

		for(int i=0; i<femphenotypes.size();i++)
			phenotypes.push_back(femphenotypes[i]);

// First traslate to scaleini
        double minel = *min_element(phenotypes.begin(),phenotypes.end());
// This is just if retrun the original maximum raw element
//        double maxrawel = *max_element(phenotypes.begin(),phenotypes.end());


        for(int i=0; i<malephenotypes.size();i++)
            malephenotypes[i]=phenotypes[i]-minel; //scale the min to be 0

        for(int i=0; i<femphenotypes.size();i++)
            femphenotypes[i]=phenotypes[femphenotypes.size()+i]-minel;

// Now put the maximum to scalefactor using the new phenotypes

		for(int i=0; i<malephenotypes.size();i++)
			phenotypes[i]=(malephenotypes[i]);

		for(int i=0; i<femphenotypes.size();i++)
			phenotypes[malephenotypes.size()+i]=(femphenotypes[i]);

        double maxel = *max_element(phenotypes.begin(),phenotypes.end());

//cout<<maxel<<endl;

		for(int i=0; i<malephenotypes.size();i++)
			malephenotypes[i]=(phenotypes[i]/maxel)*scalefactor;

		for(int i=0; i<femphenotypes.size();i++)
			femphenotypes[i]=(phenotypes[femphenotypes.size()+i]/maxel)*scalefactor;

// Check the variance is not 0 because this will impede correlation computation due to by 0 division error
double avetmep,vartmp;
        vartmp=CppTools::avevar(malephenotypes, avetmep);
        if(vartmp<=0.0){
            string erm="Problem with male phenotype variance after scaling\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);

        }
        //else cout<<"Var is "<<vartmp<<endl;
        vartmp=CppTools::avevar(femphenotypes, avetmep);
        if(vartmp<=0.0){
            string erm="Problem with male femphenotype variance after scaling\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);

        }
        //else cout<<"Var is "<<vartmp<<endl;
// Now if neccessary shift to scale ini but this will be again change the maximum. By now give an error
        if(scaleini!=0.0){
            string erm="Problem with scaling\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);
            for(int i=0; i<malephenotypes.size();i++)
                malephenotypes[i]+=scaleini;
            for(int i=0; i<femphenotypes.size();i++)
                femphenotypes[i]+=scaleini;
        }
        return maxel;
	}

// Perform a different scaling for each population:
// This function first scale the minimum to be 0, then escale the maximum to be scalefactor. Then shift to scaleini. This scaling is not just a translation so do affect to the variance
// Return the maximum raw value of each pop
	vector<double> scalephenotypeswithinpop(const int& numpops, const double& scaleini,const double& scalefactor, vector<double>& malephenotypes,vector<double>& femphenotypes, const vector<int>&pops){

		vector <vector<double> >phenotypes(numpops);
		vector<double> maxel(numpops);
		int popid,ini=0,end,range;

// now if neccessary shift to scale ini but this will be again change the maximum. By now give an error
        if(scaleini!=0.0){
            string erm="Problem with scaling within pops\n";
            cout<<erm<<endl;
            CppTools::texterr("Spectest",erm,true);

        }


        for(int p=0;p<(numpops);p++){

            popid=p+1;
            // get the range for each population

            if(popid<numpops) end=CppTools::ufindindx(pops,popid+1);
            else end=pops.size();

            range=end-ini;

            for(int i=ini; i<end;i++)
                phenotypes[p].push_back(malephenotypes[i]);

            for(int i=ini; i<end;i++)
                phenotypes[p].push_back(femphenotypes[i]);

            // First traslate to scaleini
            double minel = *min_element(phenotypes[p].begin(),phenotypes[p].end());


            for(int i=0; i<range;i++){
                malephenotypes[i+ini]=phenotypes[p][i]-minel; //scale the min to be 0

            }

            for(int i=0; i<range;i++){
                femphenotypes[i+ini]=phenotypes[p][range+i]-minel;

            }

            // Now put the maximum to scalefactor using the new phenotypes

            for(int i=0; i<range;i++)
                phenotypes[p][i]=(malephenotypes[i+ini]);

            for(int i=0; i<range;i++)
                phenotypes[p][range+i]=(femphenotypes[i+ini]);

            maxel[p] = *max_element(phenotypes[p].begin(),phenotypes[p].end());

            for(int i=0; i<range;i++)
                malephenotypes[i+ini]=(phenotypes[p][i]/maxel[p])*scalefactor;

            for(int i=0; i<range;i++)
                femphenotypes[i+ini]=(phenotypes[p][range+i]/maxel[p])*scalefactor;

            ini=end;


        } // close for numpops


        return maxel;
	}


    // This function scale the min to be scale if ini is true and scale the max on the contrary. This scaling is just a translation and do not affect variance
    void scalephenotypes(const double scale,vector<double>& malephenotypes,vector<double>& femphenotypes, bool ini){

		vector<double> phenotypes;

		for(int i=0; i<malephenotypes.size();i++)
			phenotypes.push_back(malephenotypes[i]);

		for(int i=0; i<femphenotypes.size();i++)
			phenotypes.push_back(femphenotypes[i]);

		if(ini){
            double minel = *min_element(phenotypes.begin(),phenotypes.end());
            //double maxel = *max_element(phenotypes.begin(),phenotypes.end());
            double x=scale-minel;

            for(int i=0; i<malephenotypes.size();i++)
                malephenotypes[i]=phenotypes[i]+x;

            for(int i=0; i<femphenotypes.size();i++)
                femphenotypes[i]=phenotypes[femphenotypes.size()+i]+x;
		}
		else{
            double maxel = *max_element(phenotypes.begin(),phenotypes.end());
            double y= maxel-scale;

            for(int i=0; i<malephenotypes.size();i++)
                malephenotypes[i]=phenotypes[i]-y;

            for(int i=0; i<femphenotypes.size();i++)
                femphenotypes[i]=phenotypes[femphenotypes.size()+i]-y;
		}

	}



    double computeDmax(const vector<double>& phenotypes){
        vector<double> temp =phenotypes;
        sort(temp.begin(),temp.end());
        return(fabs(temp.back()-temp.front()));
    }

    double computeDmax(const vector<double>& males,const vector<double>& females){
        double dmax;

        double maxel1 = *max_element(males.begin(),males.end());
        double maxel2 = *max_element(females.begin(),females.end());

        double minel1 = *min_element(males.begin(),males.end());
        double minel2 = *min_element(females.begin(),females.end());

        dmax = max(fabs(maxel1-minel2),fabs(maxel2-minel1));
        return dmax;
    }

// for a given mating function test for a fixed trait value in one sex a range of values in the other to see the growing behaviour through D from fixtrair val to 1 or 0
// the value of male should be lower than female
	vector<double> testfunctiongrowth(const int fid,const double& dmax,const double& maxrange, const double& V,const double& choice,const double& exponent, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){


		double fem,fixedtrait;
		double dinc=0.1;
		//double dim = 1.0+dmax/dinc;

		vector<double> temp;

		fixedtrait=maxrange-dmax;

        double drange,d=0;
        if (fixedtrait>=0) drange=dmax;
        else drange = maxrange;
        double dtimes = drange/dinc;
        int numtimes = ceil(dtimes);
//cout<<drange<<"\t"<<dinc<<"\t"<<numtimes<<endl;

		for(int id=0;id<=(numtimes);id++){
			fem=d+fixedtrait;
			temp.push_back(fun(maxrange,V,choice,exponent,fixedtrait,fem));

            d+=dinc;

		} // close for
//cout<<temp.size()<<endl;
        return(temp);

	}

	vector<double> testfunctiongrowth2(const int fid,const double& male,const double& maxrange, const double& V,const double& choice,const double& exponent, double (*fun)(const double& maxrange,const double& V,const double& choice,const double& exponent,const double& Zm,const double& Zf)){


		double fem,fixedtrait;
		double dinc=0.1;
		//double dim = 1.0+dmax/dinc;

		vector<double> temp;

		fixedtrait=male;

		for(double d=0;d<=maxrange;d+=dinc){
			fem=d;
			temp.push_back(fun(maxrange,V,choice,exponent,fixedtrait,fem));

		} // close for

        return(temp);

	}

	void testallfuncsgrowth(const double&fixtraitval, const double&V,const double& maxrange){
		bool cambia=false;
		double dmax,exponent=1.0;
		casos caso2;

		double male = fixtraitval;

        dmax = maxrange-male; // because d max depends on the fixed trait and the maxrange allowed e.g if maxtrait is 1 and fixtrait is 0.5 the maximum difference allowable is 0.5

		dmax>=male? :cambia=true; // swap only if male>dmax i.e. the maximum allowed for the fixed trait is 0.5
		if(cambia) swap(dmax,male);
//cout<<male<<"\t"<<dmax<<endl;exit(1);
		caso2.minpheno=male;

		caso2.s =V;

		string filename= "DfunTest_V"+NtoS(V)+"_FixTrait_"+NtoS(male)+".xls";
		string cabecera = "V	Choice	Male	Female	D	DD99	BD03a	Gauss	G07b	D12	FND\n";
		openExcel(filename, cabecera);

		vector<vector<double> > funs(6);

		for(double choice = -1;choice<=1.0;choice+=0.1){
		  if(fabs(choice)<0.0000001) choice =0.0;
		  caso2.inichoice=choice;

            funs[0]=(testfunctiongrowth(0,dmax,maxrange,V,choice,2*exponent,&DD99)); // get all range of function values for D for the given choice, V settings
//cout<<"here"<<endl;exit(1);
            funs[1]=(testfunctiongrowth(1,dmax,maxrange,V,choice,(2*exponent),&BD03));
            funs[2]=(testfunctiongrowth(2,dmax,maxrange,V,choice,exponent,&Gauss));
            funs[3]=(testfunctiongrowth(3,dmax,maxrange,V,choice,exponent,&G07B));
            funs[4]=(testfunctiongrowth(4,dmax,maxrange,V,choice,exponent,&D12));
            // acr 12/03/13 if FND is used the sigma insteand Va (=s^2) must be passed
            double sigma = sqrt(V)/(raizd2*fabs(choice)); // sigma=s/(sqrt(2)*C)
            funs[5]=(testfunctiongrowth(5,dmax,maxrange,sigma,choice,exponent,&FND));
/*
		for(int f=0;f<funs[4].size();f++){
				cout<<f<<"\t"<<funs[4][f]<<endl;
		}
		exit(1);
*/

		// now write
		writeExceltestgrowth(filename,caso2,funs);
/*
		for(int f=0;f<funs.size();f++){
			funs[f].clear();
			vector<double>().swap(funs[f]);
		}
*/
		}// close for choice

	}
// To generate figures similar to Figure 2 in Debarre 12. Will vary fem instead D
	void testallfuncsgrowth2(const double&fixtraitval, const double&V,const double& maxrange){
		bool cambia=false;
		double exponent=1.0;
		casos caso2;

		double male = fixtraitval;

		caso2.minpheno=male;

		caso2.s =V;

		string filename= "DfunTest2_V"+NtoS(V)+"_FixTrait_"+NtoS(male)+".xls";
		string cabecera = "V	Choice	Male	Female	D	DD99	BD03a	BD03b	Gauss	D12	CR13\n";
		openExcel(filename, cabecera);

		vector<vector<double> > funs(6);

		for(double choice = -1;choice<=1.0;choice+=0.1){
		  if(fabs(choice)<0.0000001) choice =0.0;
		  caso2.inichoice=choice;

            funs[0]=(testfunctiongrowth2(0,male,maxrange,V,choice,2*exponent,&FN2D_2c)); // get all range of function values for D for the given choice, V settings
            funs[1]=(testfunctiongrowth2(1,male,maxrange,V,choice,(2*exponent),&BD03));
            funs[2]=(testfunctiongrowth2(2,male,maxrange,V,choice,exponent,&G07B));
            funs[3]=(testfunctiongrowth2(3,male,maxrange,V,choice,exponent,&Gauss));
            funs[4]=(testfunctiongrowth2(4,male,maxrange,V,choice,exponent,&D12));
            // acr 12/03/13 if FND is used sigma instead Va must be passed
            double sigma = sqrt(V)/(raizd2*fabs(choice)); // sigma=s/(sqrt(2)*C)
            funs[5]=(testfunctiongrowth2(5,male,maxrange,sigma,choice,exponent,&FND));
/*
		for(int f=0;f<funs[1].size();f++)
				cout<<f<<"\t"<<funs[1][f]<<endl;
		exit(1);
*/
		// now write
		writeExceltestgrowth2(filename,caso2,funs);
/*
		for(int f=0;f<funs.size();f++){
			funs[f].clear();
			vector<double>().swap(funs[f]);
		}
*/
		}// close for choice

	}

	void writeExceltestgrowth(const string& filename, struct casos& caso, const vector<vector<double> >& funsvals){

		ofstream fresults; //ofstream will be opened for writing
		fresults.clear();
		fresults.open(filename.c_str(),ios_base::app);
		if(fresults.fail())
		{
			string erm="can not create output file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}

		int dcases = funsvals[0].size();

		double fem;double dist=0.0;
		for(int d=0;d<dcases;d++){
			fem=dist+caso.minpheno;
			fresults<<setprecision(2)<<caso.s<<"\t"<<setprecision(1)<<caso.inichoice<<"\t"<<caso.minpheno<<"\t"<<fem<<"\t"<<setprecision(2)<<dist<<setprecision(6)<<"\t"<<funsvals[0][d]<<"\t"<<funsvals[1][d]<<"\t"<<funsvals[2][d]<<"\t"<<funsvals[3][d]<<"\t"<<funsvals[4][d]<<"\t"<<funsvals[5][d]<<endl;
			dist+=0.1;
		}

        fresults.close();
	}

    void writeExceltestgrowth2(const string& filename, struct casos& caso, const vector<vector<double> >& funsvals){

		ofstream fresults; //ofstream will be opened for writing
		fresults.clear();
		fresults.open(filename.c_str(),ios_base::app);
		if(fresults.fail())
		{
			string erm="can not create output file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}

		double dcases = funsvals[0].size();
		double fem=0.0;double dist=0.0;
		for(int d=0;d<dcases;d++){
			dist=fabs(caso.minpheno-fem);
			fresults<<setprecision(2)<<caso.s<<"\t"<<setprecision(1)<<caso.inichoice<<"\t"<<caso.minpheno<<"\t"<<fem<<"\t"<<dist<<setprecision(6)<<"\t"<<funsvals[0][d]<<"\t"<<funsvals[1][d]<<"\t"<<funsvals[2][d]<<"\t"<<funsvals[3][d]<<"\t"<<funsvals[4][d]<<"\t"<<funsvals[5][d]<<endl;
			fem+=0.1;
		}

        fresults.close();
	}

// 30/10/13 this performs the microarea experiment by sampling mAnum microarea mean values from mx = N(aveP,sdP) and then sampling a couple for mating at each microarea.
//In each microarea the probability of mating is computed using a preference function if the mating does not occur a new couple is sampled until mating occurs
// when the matings are done in all the microareas the data is grouped and the Ipsi is computed for the population as a whole
// as a control a number of matings is also performed directly using the mAnum matigns directly from the N(aveP,sdP) population and then computing the Ipsi as before.
void justdoit(const int numruns,const int KmaxMax, double& aveP, double& sdP,double& rangeini, double& rangeend,const bool scaled,const bool proof,const double& fixtraitval,const bool traitfixed, const bool isoextreme,const int mAnum,const double& Vaini){

// initiallize seed for MonteCarlo simulations
	bool fixseed=false;
	bool scale=scaled;
	bool sini=true;
	bool send=true;

    const int numfuns=6;
    string infloop;
	// simulation parameters for the different cases
	//vector<int> PopSize;
//acr Nov13 vector of menas and vars for each microarea
    struct microarea{
        double ave;
        double var;
    }

   	vector<double> females;
   	vector<double> males;
   	vector<double> dif;
	vector<double> phenotypes;
	// for computing coefficient of variation, averages through runs etc
	vector<double> Varmale,Varfem,aveVarmale,aveVarfem,Varrest,aveVarrest;
   	vector<double> distribDD99;
   	vector<double> distribBD03a;
   	vector<double> distribBD03b;
   	vector<double> distribGauss;
   	vector<double> distribD12;
   	vector<double> distribCR13;

	vector<double> correlDD99;
   	vector<double> correlBD03a;
   	vector<double> correlBD03b;
   	vector<double> correlGauss;
   	vector<double> correlD12;
   	vector<double> correlCR13;

// Isolation index
	vector<double> IDD99;
   	vector<double> IBD03a;
   	vector<double> IBD03b;
   	vector<double> IGauss;
   	vector<double> ID12;
   	vector<double> ICR13;


// averages through replicates
	vector<double> avecorrelDD99;
   	vector<double> avecorrelBD03a;
   	vector<double> avecorrelBD03b;
   	vector<double> avecorrelGauss;
   	vector<double> avecorrelD12;
   	vector<double> avecorrelCR13;
//SD through replicates
	vector<double> sdcorrelDD99;
   	vector<double> sdcorrelBD03a;
   	vector<double> sdcorrelBD03b;
   	vector<double> sdcorrelGauss;
   	vector<double> sdcorrelD12;
   	vector<double> sdcorrelCR13;

//ACR0513 missing value counting
    vector<int> missinfDD99;
    vector<int> missinfBD03a;
    vector<int> missinfBD03b;
    vector<int> missinfGauss;
    vector<int> missinfD12;
    vector<int> missinfCR13;

    vector<int> countnDD99;
    vector<int> countnBD03a;
    vector<int> countnBD03b;
    vector<int> countnGauss;
    vector<int> countnD12;
    vector<int> countnCR13;


// averages and sd for isolationindex
// averages through replicates
	vector<double> aveIDD99;
   	vector<double> aveIBD03a;
   	vector<double> aveIBD03b;
   	vector<double> aveIGauss;
   	vector<double> aveID12;
   	vector<double> aveICR13;
//SD through replicates
	vector<double> sdIDD99;
   	vector<double> sdIBD03a;
   	vector<double> sdIBD03b;
   	vector<double> sdIGauss;
   	vector<double> sdID12;
   	vector<double> sdICR13;

    vector<double> ncorrelDD99;
   	vector<double> ncorrelBD03a;
   	vector<double> ncorrelBD03b;
   	vector<double> ncorrelGauss;
   	vector<double> ncorrelD12;
   	vector<double> ncorrelCR13;
// averages through replicates
	vector<double> avencorrelDD99;
   	vector<double> avencorrelBD03a;
   	vector<double> avencorrelBD03b;
   	vector<double> avencorrelGauss;
   	vector<double> avencorrelD12;
   	vector<double> avencorrelCR13;
//SD through replicates
	vector<double> sdncorrelDD99;
   	vector<double> sdncorrelBD03a;
   	vector<double> sdncorrelBD03b;
   	vector<double> sdncorrelGauss;
   	vector<double> sdncorrelD12;
   	vector<double> sdncorrelCR13;

    vector< vector<double> > fvals(numfuns);

    double isolini=0.05, isolend=0.95;

   	double inichoice=-1.0; // minimum choice value// is 2c-1 will vary between -1,1
   	double cinc=0.1;
   	double cmax = 1.0; // maximum choice value
   	double exponent;
   	double maxrange,meanpheno;
	//double rangeini=0.7;
	//double rangeend=1;
	double scaleini=rangeini;
	double scalefactor=rangeend; // the range produced by scalephenotypes is [scaleini,scaleini+(2/scalefactor)] the right part of the range is the maximum that depending on data could not to be reached
	//so with the default values 0,2 the range will be [0,<=1]
	int Kmaxini=10; // use Kmax only multiples fo 10
	//int Kmaxcases = 1+log10(Kmaxend/Kmaxini);
    int numchoicecases = 1 + (cmax -inichoice)/cinc;

	if(fixseed) {
		getseed2(fixseed);
		cout << "WARNING: FIXSEED MODE"<<endl;
		CppTools::textmsg2("FIXSEED MODE");
	}
	else
		getseed2(time(NULL),true);

	// measure execution time
	float start = clock(),secs;

// Generate phenotype distribution

	if(proof){ // this is just one case
		bool dmode=true;
        cout<<"\nPROOF MODE"<<endl;

		if(traitfixed){ // given a fixed trait value generate function values for the distances (dmode) or for the other sex (to get Debarre figure 2)
			//double fixtraitval=0.5;
			if(rangeini>fixtraitval)
                    rangeini=fixtraitval;
            else
                if(rangeend<fixtraitval)
                    rangeend =fixtraitval;
			maxrange=fabs(rangeend-rangeini);
			meanpheno=(rangeend+rangeini)/2.0;

            if(fabs(fixtraitval) > maxrange){
                string errmsg = "Error: Fixed trait value is out of range\n";
                cout<<errmsg<<endl;
                CppTools::texterr(errmsg,false);

                maxrange=fabs(rangeend-rangeini);
            }

			if(dmode)
                testallfuncsgrowth(fixtraitval, Vaini,maxrange);
            else
                testallfuncsgrowth2(fixtraitval, Vaini,maxrange);
		}
        else{ // generate a set of phenotypes and its function values just for checking

        caso.replicas=1;
		caso.Kmax=10;
		caso.minpheno=rangeini;
		caso.maxpheno=rangeend;
		caso.s = Vaini;
		caso.inichoice=inichoice;
		caso.endchoice=cmax;
		caso.choiceinc = cinc;
        bool putheader=true;
        phenotypes.resize(caso.Kmax);
        int indexch=0;
        maxrange=fabs(rangeend-rangeini);
        meanpheno=(rangeend+rangeini)/2.0;
        for(double choice=inichoice;choice<=cmax;choice+=cinc){

            if(fabs(choice)<0.0000001) choice =0.0;
            phenotypes=generatePhenotypes(caso.Kmax, 0.0, 1.0);

        // half males and half females
            for(int i=0;i<caso.Kmax/2;i++){
                males.push_back(phenotypes[i]);

            }
            for(int i=caso.Kmax/2;i<caso.Kmax;i++){
                females.push_back(phenotypes[i]);

            }

            if(males.size()!=females.size()){
                string errmsg="Diferent number of females "+NtoS(females.size()) + " and males "+NtoS(males.size());
                CppTools::texterr(errmsg,true);
            }

            scalephenotypes(scaleini,scalefactor,males,females);
            exponent = 1;
            (testfunction(0,fvals[0],maxrange,caso.s,choice,2*exponent,males,females,&DD99));
            (testfunction(1,fvals[1],maxrange,caso.s,choice,(2*exponent),males,females,&BD03));
            (testfunction(2,fvals[2],maxrange,caso.s,choice,exponent,males,females,&G07B));
            (testfunction(3,fvals[3],maxrange,caso.s,choice,exponent,males,females,&Gauss));
            (testfunction(4,fvals[4],maxrange,caso.s,choice,exponent,males,females,&D12));

            // acr 12/03/13 if FND is used sigma instead Va (=s^2) must be passed
            double sigma = sqrt(Vaini)/(raizd2*fabs(choice)); // sigma=s/(sqrt(2)*C)

            (testfunction(5,fvals[5],maxrange,sigma,choice,exponent,males,females,&FND));

            writefunctionval("testfunctions.xls", fvals,males,females,caso, choice,putheader);
            putheader=false;
            males.clear();
            vector<double>().swap(males);
            females.clear();
            vector<double>().swap(females);

            indexch++;

//cout<<"\nhere choice = "<<NtoS(choice)<<endl;

        } // close for choice
        } // close non d mode

	} // close if proof
	else{ // ***** AUTOMATIZED CASES *********

// ************* DEFINE MEMORY ***************************

        Varmale.resize(numchoicecases,0.0);
        Varfem.resize(numchoicecases,0.0);
        Varrest.resize(numchoicecases,0.0);

        aveVarmale.resize(numchoicecases,0.0);
        aveVarfem.resize(numchoicecases,0.0);
        aveVarrest.resize(numchoicecases,0.0);


        correlDD99.resize(numchoicecases);
        correlBD03a.resize(numchoicecases);
        correlBD03b.resize(numchoicecases);
        correlGauss.resize(numchoicecases);
        correlD12.resize(numchoicecases);
        correlCR13.resize(numchoicecases);

        IDD99.resize(numchoicecases);
        IBD03a.resize(numchoicecases);
        IBD03b.resize(numchoicecases);
        IGauss.resize(numchoicecases);
        ID12.resize(numchoicecases);
        ICR13.resize(numchoicecases);

        avecorrelDD99.resize(numchoicecases,0);
        avecorrelBD03a.resize(numchoicecases,0);
        avecorrelBD03b.resize(numchoicecases,0);
        avecorrelGauss.resize(numchoicecases,0);
        avecorrelD12.resize(numchoicecases,0);
        avecorrelCR13.resize(numchoicecases,0);

        sdcorrelDD99.resize(numchoicecases,0);
        sdcorrelBD03a.resize(numchoicecases,0);
        sdcorrelBD03b.resize(numchoicecases,0);
        sdcorrelGauss.resize(numchoicecases,0);
        sdcorrelD12.resize(numchoicecases,0);
        sdcorrelCR13.resize(numchoicecases,0);

//acr0513
        missinfDD99.resize(numchoicecases,0);
        missinfBD03a.resize(numchoicecases,0);
        missinfBD03b.resize(numchoicecases,0);
        missinfGauss.resize(numchoicecases,0);
        missinfD12.resize(numchoicecases,0);
        missinfCR13.resize(numchoicecases,0);

        countnDD99.resize(numchoicecases,0);
        countnBD03a.resize(numchoicecases,0);
        countnBD03b.resize(numchoicecases,0);
        countnGauss.resize(numchoicecases,0);
        countnD12.resize(numchoicecases,0);
        countnCR13.resize(numchoicecases,0);

        aveIDD99.resize(numchoicecases,0);
        aveIBD03a.resize(numchoicecases,0);
        aveIBD03b.resize(numchoicecases,0);
        aveIGauss.resize(numchoicecases,0);
        aveID12.resize(numchoicecases,0);
        aveICR13.resize(numchoicecases,0);

        sdIDD99.resize(numchoicecases,0);
        sdIBD03a.resize(numchoicecases,0);
        sdIBD03b.resize(numchoicecases,0);
        sdIGauss.resize(numchoicecases,0);
        sdID12.resize(numchoicecases,0);
        sdICR13.resize(numchoicecases,0);

    // normalized probabilities
        ncorrelDD99.resize(numchoicecases);
        ncorrelBD03a.resize(numchoicecases);
        ncorrelBD03b.resize(numchoicecases);
        ncorrelGauss.resize(numchoicecases);
        ncorrelD12.resize(numchoicecases);
        ncorrelCR13.resize(numchoicecases);

        avencorrelDD99.resize(numchoicecases,0);
        avencorrelBD03a.resize(numchoicecases,0);
        avencorrelBD03b.resize(numchoicecases,0);
        avencorrelGauss.resize(numchoicecases,0);
        avencorrelD12.resize(numchoicecases,0);
        avencorrelCR13.resize(numchoicecases,0);

        sdncorrelDD99.resize(numchoicecases,0);
        sdncorrelBD03a.resize(numchoicecases,0);
        sdncorrelBD03b.resize(numchoicecases,0);
        sdncorrelGauss.resize(numchoicecases,0);
        sdncorrelD12.resize(numchoicecases,0);
        sdncorrelCR13.resize(numchoicecases,0);

		string filename="SpecR"+NtoS(numruns)+"_Cost"+NtoS(maxiterations)+"_";
        if(unbiasedcorrel) filename+="ur_";
        else filename+="r_";
		if(scale){
            if(sini) filename+=("Scal(I"+NtoS(rangeini)+")_");
            if(send) filename+=("Scal(E"+NtoS(rangeend)+")_");

		}

		if(traitfixed) filename+="Fix"+NtoS(fixtraitval)+"_";



string cabecera;
if(isoextreme)
    cabecera="Choice	Va	Kmax	traitmin	traitmax	Dmax	CVFem	CVMale\tVarDif	rDD99	SDrDD99\tMissingVals\trBD03a	SDrBD03a	MissingVals\trG07B	SDrG07B	MissingVals\trGauss	SDrGauss	MissingVals\trD12	SDrD12	MissingVals\trFND	SDrFND\tMissingVals\tNrDD99	SDNrDD99	NrBD03a\tSDNrBD03a	NrG07B	SDNrG07B	NrGauss	SDNrGauss\tNrD12\tSDNrD12	NrFND	SDNrFND	IDD99	SDIDD99\tIBD03a	SDIBD03a	IG07B	SDIG07B\tIGauss	SDIGauss	ID12	SDID12	IFND	SDIFND	(num of runs)\n";
else
    cabecera="Choice	Va	Kmax	traitmin	traitmax	Dmax	CVFem	CVMale\tVarDif	rDD99	SDrDD99\tMissingVals\trBD03a	SDrBD03a	MissingVals\trG07B	SDrG07B	MissingVals\trGauss	SDrGauss	MissingVals\trD12	SDrD12	MissingVals\trFND	SDrFND\tMissingVals\tNrDD99	SDNrDD99	NrBD03a\tSDNrBD03a	NrG07B	SDNrG07B	NrGauss	SDNrGauss\tNrD12\tSDNrD12	NrFND	SDNrFND	ImedDD99	SDImedDD99\tImedBD03a	SDImedBD03a	ImedG07B	SDImedG07B\tImedGauss	SDImedGauss	ImedD12	SDImedD12	ImedFND	SDImedFND	(num of runs)\n";

// checking phenotype distribution
//double aveavep=0.0,aveavem=0.0,aveavef=0.0;
double avep=0.0,avem=0.0,avef=0.0,avedif=0.0;

        if(scale){
            if(sini && send){
                maxrange = fabs(rangeend-rangeini); //Dmax
                meanpheno=(rangeend+rangeini)/2.0;
                scaleini=rangeini; //for scaling phenotypes
                scalefactor = rangeend;
                if(isoextreme){
                    isolend=maxrange-isolini;
                    if(isolend<=(maxrange*.75)) isolend=maxrange-(isolini/10.0);
                    filename+="Iextr_";
                }
                else{ //ACR0513
                    isolend=(maxrange)/2.0;
                    filename+="Imed_";
                }
//cout<<scaleini<<"\t"<<scalefactor<<endl;
                while(isolend<=isolini){isolini/=2;}
            }

            if(traitfixed)
                if(fabs(fixtraitval) > maxrange){
                    string errmsg = "Error: Fixed trait value is out of range\n";
                    cout<<errmsg<<endl;
                    CppTools::texterr(errmsg,true);
                }
        }
		filename+=("N("+NtoS(aveP)+","+NtoS(sdP)+")_Isol("+NtoS(isolini)+", "+NtoS(isolend)+")_Vaini("+NtoS(Vaini)+").xls");
		openExcel(filename,cabecera); // las referencias deben inicializarse a la vez que se declaran pq son punteros constantes
// ******* GENERATE CASES FOR Va and Kmax
        double Va=Vaini; // Vaini*=2 increased at the end of the for the cases are 0.01, 0.02, 0.04, 0.08, 0.16, 0.32, 0.64
        int Kmax;
        int NVcases=7;

        for(int iVa=0;iVa<NVcases;iVa++){

        Kmax=Kmaxini;

		for(int iKmax=1;iKmax<KmaxMax;iKmax++){
            switch(iKmax){
                case 1: Kmax*=1;break; //kmax = 10
                case 2: Kmax*=5;break; //kmax = 50
                case 3: Kmax*=2;break; //kmax = 100
                case 4: Kmax*=10;break; //kmax = 1000
                default:
                    string errmsg = "Error: in Kmax the kmax end was "+NtoS(KmaxMax)+"\n";
                    cout<<errmsg<<endl;
                    CppTools::texterr(errmsg,true);
                    break;
            }
        //string CASO = "Runs: "+NtoS(numruns)+". Dmax:"+NtoS(maxrange)+ "("+NtoS(rangeini)+", "+NtoS(rangeend)+")." + " Va: "+NtoS(Va)+" Choice range: ("+NtoS(inichoice)+" -" +NtoS(cmax)+") with increases of "+NtoS(cinc);

		caso.replicas=numruns;
		caso.Kmax=Kmax;

		caso.s = Va;
		caso.inichoice=inichoice;
		caso.endchoice=cmax;
		caso.choiceinc = cinc;

        phenotypes.resize(Kmax);
        if(Kmax%2!=0){
            string errmsg = "Error: please use an even Kmax current value was "+NtoS(Kmax)+"\n";
            cout<<errmsg<<endl;
            CppTools::texterr(errmsg,true);
        }
        males.resize(Kmax/2);

        females.resize(males.size());
        dif.resize(males.size());
        distribDD99.resize(males.size());
        distribBD03a.resize(males.size());
        distribBD03b.resize(males.size());
        distribGauss.resize(males.size());
        distribD12.resize(males.size());
        distribCR13.resize(males.size());



        if(traitfixed)
            for(int i=0;i<males.size();i++){
                males[i]=(fixtraitval);

            }
// ************* PERFORM DIFFERENT RUNS ***************************
        for(int run=0;run<numruns;run++){
// perform cases for the different choices
        int indexch=0;
        for(double choice=inichoice;choice<=cmax;choice+=cinc){

            if(fabs(choice)<0.0000001) choice =0.0;

            //phenotypes=generatePhenotypes(Kmax, aveP, sdP); //remember that by default sdP=rangemax=Dmax
            generatePhenotypes(phenotypes, aveP, sdP);
            //generatePhenotypes(phenotypes, aveP, 0.0);
        // half males and half females

            if(!traitfixed)
                for(int i=0;i<males.size();i++){
                    males[i]=(phenotypes[i]);
                }
            int count=males.size();
            for(int i=0;i<females.size();i++){
                females[i]=(phenotypes[count+i]);

            }

            for(int i=0;i<males.size();i++){
//cout<<males[i]<<"\t"<<females[i]<<endl;
                dif[i]=(males[i]-females[i]);
            }

            if(males.size()!=females.size()){
                string errmsg="Diferent number of females "+NtoS(females.size()) + " and males "+NtoS(males.size());
                CppTools::texterr(errmsg,true);
            }
//writephenotypes("testfenotipos.txt", males,females);

        // scale phenotypes use rangeini and rangeend to compute scaleini and scalefactor following
        //scaleini=rangeini, scalefactor = 2/(rangeend-scaleini) to get [scaleini,scaleini+(2/scalefactor)] =[rangeini, rangeend]
            if(scale &&sini&&send){ //by default
                scalephenotypes(scaleini,scalefactor,males,females);
/*
for(int i=0;i<males.size();i++)
    cout<<males[i]<<"\t"<<females[i]<<endl;
exit(1);
*/
            }

            else{
                if(scale){
                    if(sini){
                        rangeend = *max_element(phenotypes.begin(),phenotypes.end());
                        scaleini=rangeini;
                    }
                    else{
                        rangeini = *min_element(phenotypes.begin(),phenotypes.end());
                        scaleini=rangeend;
                    }

                    scalephenotypes(scaleini,males,females,sini);
                }
                else{
                    rangeini = *min_element(phenotypes.begin(),phenotypes.end());
                    rangeend = *max_element(phenotypes.begin(),phenotypes.end());
                }
                maxrange = fabs(rangeend-rangeini); //Dmax
                meanpheno=(rangeend+rangeini)/2.0;

 // Solo afecta si no hay escaling en ambos lados PERO en ese caso hay que recalcular los extremos para I
                if(isoextreme){
                    isolend=maxrange-isolini;
                    filename+="Iextr_";
                }
                else{ //ACR0513
                    isolend=(maxrange)/2.0;
                    filename+="Imed_";
                }
//cout<<rangeend<<"\t"<<rangeini<<"\t"<<maxrange<<endl;
//exit(1);
                if(traitfixed)
                    if(fabs(fixtraitval) > maxrange){
                        string errmsg = "Error: Fixed trait value is out of range\n";
                        cout<<errmsg<<endl;
                        CppTools::texterr(errmsg,true);
                    }
            } // close else not scaleini & end

            caso.minpheno=rangeini;
            caso.maxpheno=rangeend;
//cout<<rangeend<<"\t"<<rangeini<<"\t"<<maxrange<<endl;
//cout<<caso.maxpheno<<"\t"<<caso.minpheno<<"\t"<<fabs(caso.maxpheno-caso.minpheno)<<endl;
//if(choice>=0.06) exit(1);
            if(traitfixed)
                Varmale[indexch]=0.0; // compute variation coefficient in male phenotypes for echa choice case
            else
                Varmale[indexch]=sqrt(CppTools::avevar(males, avem))/avem; // compute variation coefficient in male phenotypes for echa choice case

            Varfem[indexch]=sqrt(CppTools::avevar(females, avef))/avef; // compute variation coefficient in male phenotypes for echa choice case

            Varrest[indexch]=(CppTools::avevar(dif, avedif));

//cout<<"medias "<<"\t"<<avef<<"\t"<<avem<<endl;

/*
if(indexch==(1))
if(Kmax>10){
if(Kmax>100){cout<<(aveavep/(numruns))<<"\t"<<(aveavem/(numruns))<<"\t"<<(aveavef/(numruns))<<endl;exit(1);}

CppTools::avevar(phenotypes, avep);
CppTools::avevar(males, avem);
CppTools::avevar(females, avef);
aveavep+=avep;
aveavem+=avem;
aveavef+=avef;
}
*/

//writephenotypes("testfenotipos.txt", males,females);
//exit(2);
        // compute and store distributions for each of the six methods

            exponent = 1;
            double sigma = sqrt(Va);

            // acr 0513
            if(choice!=0) sigma/=(raizd2*fabs(choice)); // sigma=s/(sqrt(2)*C)

            if(choice!=0){

                computedistrib(distribDD99,maxrange,Va,choice,2*exponent,males,females,&DD99);

                computedistrib(distribBD03a,maxrange,Va,choice,2*exponent,males,females,&BD03);

                computedistrib(distribBD03b,maxrange,Va,choice,exponent,males,females,&G07B);

                computedistrib(distribGauss,maxrange,Va,choice,exponent,males,females,&Gauss);

                computedistrib(distribD12,maxrange,Va,choice,exponent,males,females,&D12);

                // acr 12/03/13 if FND is used sigma instead of Va (=s^2) must be precomputed and passed

                computedistrib(distribCR13,maxrange,sigma,choice,exponent,males,females,&FND);
            }

        // perform Kmax/2 matings using the value of the functions or the normalized probabilities
        //store the correlation coefficient between mated phenotypes
        // without normalization


            correlDD99[indexch]=(mating(caso,run,infloop,0,maxrange,Va,choice,2*exponent,males,females,&DD99));


            correlBD03a[indexch]=(mating(caso,run,infloop,1,maxrange,Va,choice,2*exponent,males,females,&BD03));

            correlBD03b[indexch]=(mating(caso,run,infloop,2,maxrange,Va,choice,exponent,males,females,&G07B));

            correlGauss[indexch]=(mating(caso,run,infloop,3,maxrange,Va,choice,exponent,males,females,&Gauss));

            correlD12[indexch]=(mating(caso,run,infloop,4,maxrange,Va,choice,exponent,males,females,&D12));

             // acr 12/03/13 if FND is used sigma instead of Va (=s^2) must be precomputed and passed
            correlCR13[indexch]=(mating(caso,run,infloop,5,maxrange,sigma,choice,exponent,males,females,&FND));


//if(errflag){exit(1);} // el ultimo argumento indica el tipo de funcion para el que ocurre el problema


// Isolation index  ACR0513 el indice I no varia entre replicas
if(run==0){
            IDD99[indexch]=(Isolation(0,maxrange,isolini,isolend,Va,choice,2*exponent,&DD99));
            IBD03a[indexch]=(Isolation(1,maxrange,isolini,isolend,Va,choice,2*exponent,&BD03));
            IBD03b[indexch]=(Isolation(2,maxrange,isolini,isolend,Va,choice,exponent,&G07B));
            IGauss[indexch]=(Isolation(3,maxrange,isolini,isolend,Va,choice,exponent,&Gauss));
            ID12[indexch]=(Isolation(4,maxrange,isolini,isolend,Va,choice,exponent,&D12));
            ICR13[indexch]=(Isolation(5,maxrange,isolini,isolend,sigma,choice,exponent,&FND));
}
        // ACUMS THROUG RUNS
            aveVarmale[indexch]+=Varmale[indexch];
            aveVarfem[indexch]+=Varfem[indexch];
            aveVarrest[indexch]+=Varrest[indexch];
        // acr 0513
            if(fabs(correlDD99[indexch])<=1.0){
                avecorrelDD99[indexch]+=correlDD99[indexch];
                sdcorrelDD99[indexch]+=(correlDD99[indexch]*correlDD99[indexch]);
            }

            else{
                missinfDD99[indexch]+=1;

            }

            if(fabs(correlBD03a[indexch])<=1.0){
                avecorrelBD03a[indexch]+=correlBD03a[indexch];
                sdcorrelBD03a[indexch]+=(correlBD03a[indexch]*correlBD03a[indexch]);
            }

            else{
                missinfBD03a[indexch]+=1;
            }

            if(fabs(correlBD03b[indexch])<=1.0){
                avecorrelBD03b[indexch]+=correlBD03b[indexch];
                sdcorrelBD03b[indexch]+=(correlBD03b[indexch]*correlBD03b[indexch]);
            }

            else{
                missinfBD03b[indexch]+=1;
            }

            if(fabs(correlGauss[indexch])<=1.0){
                avecorrelGauss[indexch]+=correlGauss[indexch];
                sdcorrelGauss[indexch]+=(correlGauss[indexch]*correlGauss[indexch]);
            }

            else{
                missinfGauss[indexch]+=1;
            }

            if(fabs(correlD12[indexch])<=1.0){
                avecorrelD12[indexch]+=correlD12[indexch];
                sdcorrelD12[indexch]+=(correlD12[indexch]*correlD12[indexch]);
            }

            else{
                missinfD12[indexch]+=1;
            }

            if(fabs(correlCR13[indexch])<=1.0){
                avecorrelCR13[indexch]+=correlCR13[indexch];
                sdcorrelCR13[indexch]+=(correlCR13[indexch]*correlCR13[indexch]);
            }

            else{
                missinfCR13[indexch]+=1;
            }


            // Isolation index
 /*  ACR0513 el indice I no varia entre replicas
            aveIDD99[indexch]+=IDD99[indexch];
            aveIBD03a[indexch]+=IBD03a[indexch];
            aveIBD03b[indexch]+=IBD03b[indexch];
            aveIG07[indexch]+=IG07[indexch];
            aveID12[indexch]+=ID12[indexch];
            aveICR13[indexch]+=ICR13[indexch];

            sdIDD99[indexch]+=(IDD99[indexch]*IDD99[indexch]);
            sdIBD03a[indexch]+=(IBD03a[indexch]*IBD03a[indexch]);
            sdIBD03b[indexch]+=(IBD03b[indexch]*IBD03b[indexch]);
            sdIG07[indexch]+=(IG07[indexch]*IG07[indexch]);
            sdID12[indexch]+=(ID12[indexch]*ID12[indexch]);
            sdICR13[indexch]+=(ICR13[indexch]*ICR13[indexch]);
*/

         // normalized probabilities

            ncorrelDD99[indexch]=(mating(0,maxrange,Va,choice,2*exponent,males,females,distribDD99,&DD99));

            ncorrelBD03a[indexch]=(mating(1,maxrange,Va,choice,2*exponent,males,females,distribBD03a,&BD03));
            ncorrelBD03b[indexch]=(mating(2,maxrange,Va,choice,exponent,males,females,distribBD03b,&G07B));
            ncorrelGauss[indexch]=(mating(3,maxrange,Va,choice,exponent,males,females,distribGauss,&Gauss));
            ncorrelD12[indexch]=(mating(4,maxrange,Va,choice,exponent,males,females,distribD12,&D12));
         // acr 12/03/13 if FND is used sigma instead of Va (=s^2) must be precomputed and passed
            ncorrelCR13[indexch]=(mating(5,maxrange,sigma,choice,exponent,males,females,distribCR13,&FND));

//if(errflag){writeCaso("casoNorm.txt",caso,funflag);} // el ultimo argumento indica el tipo de funcion para el que ocurre el problema

        // ACUMS THROUG RUNS
            if(fabs(ncorrelDD99[indexch])<=1.0){
                avencorrelDD99[indexch]+=ncorrelDD99[indexch];
                sdncorrelDD99[indexch]+=(ncorrelDD99[indexch]*ncorrelDD99[indexch]);

                countnDD99[indexch]+=1;
            }

            if(fabs(ncorrelBD03a[indexch])<=1.0){
                avencorrelBD03a[indexch]+=ncorrelBD03a[indexch];
                 sdncorrelBD03a[indexch]+=(ncorrelBD03a[indexch]*ncorrelBD03a[indexch]);
                 countnBD03a[indexch]+=1;
            }

            if(fabs(ncorrelBD03b[indexch])<=1.0){
                avencorrelBD03b[indexch]+=ncorrelBD03b[indexch];
                sdncorrelBD03b[indexch]+=(ncorrelBD03b[indexch]*ncorrelBD03b[indexch]);
                countnBD03b[indexch]+=1;
            }

            if(fabs(ncorrelGauss[indexch])<=1.0){
                avencorrelGauss[indexch]+=ncorrelGauss[indexch];
                sdncorrelGauss[indexch]+=(ncorrelGauss[indexch]*ncorrelGauss[indexch]);
                countnGauss[indexch]+=1;
            }

            if(fabs(ncorrelD12[indexch])<=1.0){
                avencorrelD12[indexch]+=ncorrelD12[indexch];
                sdncorrelD12[indexch]+=(ncorrelD12[indexch]*ncorrelD12[indexch]);
                countnD12[indexch]+=1;
            }

            if(fabs(ncorrelCR13[indexch])<=1.0){
                avencorrelCR13[indexch]+=ncorrelCR13[indexch];
                sdncorrelCR13[indexch]+=(ncorrelCR13[indexch]*ncorrelCR13[indexch]);
                countnCR13[indexch]+=1;
            }


            // clean vectors

 /*           if(!traitfixed){
                males.clear();
                vector<double>().swap(males);
            }
            females.clear();
            vector<double>().swap(females);
            dif.clear();
            vector<double>().swap(dif);
*/
            indexch++;

//cout<<"\nhere choice = "<<NtoS(choice)<<endl;

        } // close for choice

//cout<<"Run "<<run<<" finished"<<endl;
        } // close for numruns

        //Compute STATISTICS for runs
        for(int ch=0;ch<numchoicecases;ch++){

            aveVarmale[ch]/=numruns;
            aveVarfem[ch]/=numruns;
            aveVarrest[ch]/=numruns;
//ACR0513
//only count runs with no missing values
            if(missinfDD99[ch]<numruns){
                avecorrelDD99[ch]/=(numruns-missinfDD99[ch]);
                sdcorrelDD99[ch]/=(numruns-missinfDD99[ch]);
            }

            if(missinfBD03a[ch]<numruns){
                avecorrelBD03a[ch]/=(numruns-missinfBD03a[ch]);
                sdcorrelBD03a[ch]/=(numruns-missinfBD03a[ch]);
            }

            if(missinfBD03b[ch]<numruns){
                avecorrelBD03b[ch]/=(numruns-missinfBD03b[ch]);
                sdcorrelBD03b[ch]/=(numruns-missinfBD03b[ch]);
            }

            if(missinfGauss[ch]<numruns){
                avecorrelGauss[ch]/=(numruns-missinfGauss[ch]);
                sdcorrelGauss[ch]/=(numruns-missinfGauss[ch]);
            }

            if(missinfD12[ch]<numruns){
                avecorrelD12[ch]/=(numruns-missinfD12[ch]);
                sdcorrelD12[ch]/=(numruns-missinfD12[ch]);
            }

            if(missinfCR13[ch]<numruns){
                avecorrelCR13[ch]/=(numruns-missinfCR13[ch]);
                sdcorrelCR13[ch]/=(numruns-missinfCR13[ch]);
            }


            sdcorrelDD99[ch] = sqrt(fabs(sdcorrelDD99[ch] - (avecorrelDD99[ch]*avecorrelDD99[ch])));
            sdcorrelBD03a[ch] = sqrt(fabs(sdcorrelBD03a[ch] - (avecorrelBD03a[ch]*avecorrelBD03a[ch])));
            sdcorrelBD03b[ch] = sqrt(fabs(sdcorrelBD03b[ch] - (avecorrelBD03b[ch]*avecorrelBD03b[ch])));
            sdcorrelGauss[ch] = sqrt(fabs(sdcorrelGauss[ch] - (avecorrelGauss[ch]*avecorrelGauss[ch])));
            sdcorrelD12[ch] = sqrt(fabs(sdcorrelD12[ch] - (avecorrelD12[ch]*avecorrelD12[ch])));
            sdcorrelCR13[ch] = sqrt(fabs(sdcorrelCR13[ch] - (avecorrelCR13[ch]*avecorrelCR13[ch])));
/* ACR0513 el indice I no varia entre replicas
            sdIDD99[ch]/=numruns;
            sdIBD03a[ch]/=numruns;
            sdIBD03b[ch]/=numruns;
            sdIG07[ch]/=numruns;
            sdID12[ch]/=numruns;
            sdICR13[ch]/=numruns;

            sdIDD99[ch] = sqrt(fabs(sdIDD99[ch] - (aveIDD99[ch]*aveIDD99[ch])));
            sdIBD03a[ch] = sqrt(fabs(sdIBD03a[ch] - (aveIBD03a[ch]*aveIBD03a[ch])));
            sdIBD03b[ch] = sqrt(fabs(sdIBD03b[ch] - (aveIBD03b[ch]*aveIBD03b[ch])));
            sdIG07[ch] = sqrt(fabs(sdIG07[ch] - (aveIG07[ch]*aveIG07[ch])));
            sdID12[ch] = sqrt(fabs(sdID12[ch] - (aveID12[ch]*aveID12[ch])));
            sdICR13[ch] = sqrt(fabs(sdICR13[ch] - (aveICR13[ch]*aveICR13[ch])));
*/
//  ACR0513 el indice I no varia entre replicas. Aprovecho el acumulador pero en la implementacio actual no es necesario podria usarse directamente IDD99 etc en vez de aveIDD99
            aveIDD99[ch]=IDD99[ch];
            aveIBD03a[ch]=IBD03a[ch];
            aveIBD03b[ch]=IBD03b[ch];
            aveIGauss[ch]=IGauss[ch];
            aveID12[ch]=ID12[ch];
            aveICR13[ch]=ICR13[ch];


          // normalized probs
            if(countnDD99[ch]>0){
                avencorrelDD99[ch]/=(countnDD99[ch]);
                sdncorrelDD99[ch]/=(countnDD99[ch]);
            }
            if(countnBD03a[ch]>0){
                avencorrelBD03a[ch]/=(countnBD03a[ch]);
                sdncorrelBD03a[ch]/=(countnBD03a[ch]);
            }

            if(countnBD03b[ch]>0){
                avencorrelBD03b[ch]/=(countnBD03b[ch]);
                sdncorrelBD03b[ch]/=(countnBD03b[ch]);
            }

            if(countnGauss[ch]>0){
                avencorrelGauss[ch]/=(countnGauss[ch]);
                sdncorrelGauss[ch]/=(countnGauss[ch]);
            }

            if(countnD12[ch]>0){
                avencorrelD12[ch]/=(countnD12[ch]);
                sdncorrelD12[ch]/=(countnD12[ch]);
            }
            if(countnCR13[ch]>0){
                avencorrelCR13[ch]/=(countnCR13[ch]);
                sdncorrelCR13[ch]/=(countnCR13[ch]);
            }


            sdncorrelDD99[ch] = sqrt(fabs(sdncorrelDD99[ch] - (avencorrelDD99[ch]*avencorrelDD99[ch])));
            sdncorrelBD03a[ch] = sqrt(fabs(sdncorrelBD03a[ch] - (avencorrelBD03a[ch]*avencorrelBD03a[ch])));
            sdncorrelBD03b[ch] = sqrt(fabs(sdncorrelBD03b[ch] - (avencorrelBD03b[ch]*avencorrelBD03b[ch])));
            sdncorrelGauss[ch] = sqrt(fabs(sdncorrelGauss[ch] - (avencorrelGauss[ch]*avencorrelGauss[ch])));
            sdncorrelD12[ch] = sqrt(fabs(sdncorrelD12[ch] - (avencorrelD12[ch]*avencorrelD12[ch])));
            sdncorrelCR13[ch] = sqrt(fabs(sdncorrelCR13[ch] - (avencorrelCR13[ch]*avencorrelCR13[ch])));
        } // close for for computing statistics



// ********************* OUTPUT ************************************
//cout<<"\nKmaxprev "<<Kmax<<endl;

		writeExcel(filename, caso, aveVarmale,aveVarfem,aveVarrest,avecorrelDD99, avecorrelBD03a, avecorrelBD03b, avecorrelGauss,avecorrelD12,avecorrelCR13, sdcorrelDD99, sdcorrelBD03a, sdcorrelBD03b, sdcorrelGauss,  sdcorrelD12, sdcorrelCR13,avencorrelDD99, avencorrelBD03a, avencorrelBD03b, avencorrelGauss, avencorrelD12, avencorrelCR13, sdncorrelDD99, sdncorrelBD03a,  sdncorrelBD03b, sdncorrelGauss, sdncorrelD12, sdncorrelCR13, aveIDD99, aveIBD03a, aveIBD03b, aveIGauss,aveID12,aveICR13, sdIDD99, sdIBD03a, sdIBD03b, sdIGauss,  sdID12, sdICR13,missinfDD99,missinfBD03a,missinfBD03b,missinfGauss,missinfD12,missinfCR13);
//cout<<"\nKmaxPost "<<Kmax<<endl;

// **** REINITIALIZE ACUMS
        aveVarmale.assign(numchoicecases,0);
        aveVarfem.assign(numchoicecases,0);
        aveVarrest.assign(numchoicecases,0);

        avecorrelDD99.assign(numchoicecases,0);
        avecorrelBD03a.assign(numchoicecases,0);
        avecorrelBD03b.assign(numchoicecases,0);
        avecorrelGauss.assign(numchoicecases,0);
        avecorrelD12.assign(numchoicecases,0);
        avecorrelCR13.assign(numchoicecases,0);

        missinfDD99.assign(numchoicecases,0);
        missinfBD03a.assign(numchoicecases,0);
        missinfBD03b.assign(numchoicecases,0);
        missinfGauss.assign(numchoicecases,0);
        missinfD12.assign(numchoicecases,0);
        missinfCR13.assign(numchoicecases,0);

        countnDD99.assign(numchoicecases,0);
        countnBD03a.assign(numchoicecases,0);
        countnBD03b.assign(numchoicecases,0);
        countnGauss.assign(numchoicecases,0);
        countnD12.assign(numchoicecases,0);
        countnCR13.assign(numchoicecases,0);

        sdcorrelDD99.assign(numchoicecases,0);
        sdcorrelBD03a.assign(numchoicecases,0);
        sdcorrelBD03b.assign(numchoicecases,0);
        sdcorrelGauss.assign(numchoicecases,0);
        sdcorrelD12.assign(numchoicecases,0);
        sdcorrelCR13.assign(numchoicecases,0);

        avencorrelDD99.assign(numchoicecases,0);
        avencorrelBD03a.assign(numchoicecases,0);
        avencorrelBD03b.assign(numchoicecases,0);
        avencorrelGauss.assign(numchoicecases,0);
        avencorrelD12.assign(numchoicecases,0);
        avencorrelCR13.assign(numchoicecases,0);

        sdncorrelDD99.assign(numchoicecases,0);
        sdncorrelBD03a.assign(numchoicecases,0);
        sdncorrelBD03b.assign(numchoicecases,0);
        sdncorrelGauss.assign(numchoicecases,0);
        sdncorrelD12.assign(numchoicecases,0);
        sdncorrelCR13.assign(numchoicecases,0);

        aveIDD99.assign(numchoicecases,0);
        aveIBD03a.assign(numchoicecases,0);
        aveIBD03b.assign(numchoicecases,0);
        aveIGauss.assign(numchoicecases,0);
        aveID12.assign(numchoicecases,0);
        aveICR13.assign(numchoicecases,0);

        sdIDD99.assign(numchoicecases,0);
        sdIBD03a.assign(numchoicecases,0);
        sdIBD03b.assign(numchoicecases,0);
        sdIGauss.assign(numchoicecases,0);
        sdID12.assign(numchoicecases,0);
        sdICR13.assign(numchoicecases,0);

   //     phenotypes.clear();
    //    vector<double>().swap(phenotypes);
/*
        if(traitfixed){
            males.clear();
            vector<double>().swap(males);
            dif.clear();
            vector<double>().swap(dif);
        }
        */
	} // close for Kmax cases

	//if(Va>0.25) Va=0.5;
	Va*=2;
//cout<<"\nVa "<<Va<<endl;
	} // close for Va
//cout<<"\nhere out "<<endl;

//string fSPtest = "SPECtest("+NtoS(rangeini)+"_"+NtoS(rangeend)+").txt";
//CppTools::textmsg3(fSPtest,infloop);

// ****************** FREED MEMORY
        aveVarmale.clear();
        aveVarfem.clear();
        aveVarrest.clear();

        avecorrelDD99.clear();
        avecorrelBD03a.clear();
        avecorrelBD03b.clear();
        avecorrelGauss.clear();
        avecorrelD12.clear();
        avecorrelCR13.clear();

        missinfDD99.clear();
        missinfBD03a.clear();
        missinfBD03b.clear();
        missinfGauss.clear();
        missinfD12.clear();
        missinfCR13.clear();


        sdcorrelDD99.clear();
        sdcorrelBD03a.clear();
        sdcorrelBD03b.clear();
        sdcorrelGauss.clear();
        sdcorrelD12.clear();
        sdcorrelCR13.clear();

        avencorrelDD99.clear();
        avencorrelBD03a.clear();
        avencorrelBD03b.clear();
        avencorrelGauss.clear();
        avencorrelD12.clear();
        avencorrelCR13.clear();

        sdncorrelDD99.clear();
        sdncorrelBD03a.clear();
        sdncorrelBD03b.clear();
        sdncorrelGauss.clear();
        sdncorrelD12.clear();
        sdncorrelCR13.clear();

        vector<double>().swap(avecorrelDD99);
        vector<double>().swap(avecorrelBD03a);
        vector<double>().swap(avecorrelBD03b);
        vector<double>().swap(avecorrelGauss);
        vector<double>().swap(avecorrelD12);
        vector<double>().swap(avecorrelCR13);

        vector<double>().swap(sdcorrelDD99);
        vector<double>().swap(sdcorrelBD03a);
        vector<double>().swap(sdcorrelBD03b);
        vector<double>().swap(sdcorrelGauss);
        vector<double>().swap(sdcorrelD12);
        vector<double>().swap(sdcorrelCR13);

       vector<double>().swap( avencorrelDD99);
        vector<double>().swap(avencorrelBD03a);
        vector<double>().swap(avencorrelBD03b);
        vector<double>().swap(avencorrelGauss);
        vector<double>().swap(avencorrelD12);
        vector<double>().swap(avencorrelCR13);

        vector<double>().swap(sdncorrelDD99);
        vector<double>().swap(sdncorrelBD03a);
        vector<double>().swap(sdncorrelBD03b);
        vector<double>().swap(sdncorrelGauss);
        vector<double>().swap(sdncorrelD12);
        vector<double>().swap(sdncorrelCR13);


	} // close else of the if(proof)

	    secs = (double)(clock() - start) / CLOCKS_PER_SEC;
		cout<<"\n\nSpecTest FINISHED. ";
		if(secs>60.0){
			secs/=60.0;
			cout<<"Time processing: "<<secs<<" minutes"<<endl;
		}
		else cout<<"Time processing: "<<secs<<" seconds"<<endl;


	} // close justdoit


	void openExcel(const string& filename, const string& cabecera){
		ofstream fresults; //ofstream will be opened for writing
		fresults.clear();
		fresults.open(filename.c_str(),ios_base::app);
		if(fresults.fail())
		{
			string erm="can not create output file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}

        fresults<<cabecera;


	}
	void writeExcel(const string& filename, struct casos& caso, const vector<double>& aveVarmale,const vector<double>& aveVarfem, const vector<double>& aveVarrest,const vector<double>& avecorrelDD99, const vector<double>& avecorrelBD03a, const vector<double>& avecorrelBD03b, const vector<double>& avecorrelGauss, const vector<double>& avecorrelD12, const vector<double>& avecorrelCR13, const vector<double>& sdcorrelDD99, const vector<double>& sdcorrelBD03a, const vector<double>& sdcorrelBD03b, const vector<double>& sdcorrelGauss, const vector<double>& sdcorrelD12, const vector<double>& sdcorrelCR13 ,const vector<double>& avencorrelDD99, const vector<double>& avencorrelBD03a, const vector<double>& avencorrelBD03b, const vector<double>& avencorrelGauss, const vector<double>& avencorrelD12, const vector<double>& avencorrelCR13, const vector<double>& sdncorrelDD99, const vector<double>& sdncorrelBD03a, const vector<double>& sdncorrelBD03b, const vector<double>& sdncorrelGauss, const vector<double>& sdncorrelD12, const vector<double>& sdncorrelCR13, const vector<double>& aveIDD99, const vector<double>& aveIBD03a, const vector<double>& aveIBD03b, const vector<double>& aveIGauss,const vector<double>& aveID12,const vector<double>& aveICR13, const vector<double>& sdIDD99, const vector<double>& sdIBD03a, const vector<double>& sdIBD03b, const vector<double>& sdIGauss,  const vector<double>& sdID12, const vector<double>& sdICR13,const vector<int>& missinfDD99,const vector<int>& missinfBD03a,const vector<int>& missinfBD03b,const vector<int>& missinfGauss,const vector<int>& missinfD12,const vector<int>& missinfCR13){

		ofstream fresults; //ofstream will be opened for writing
		fresults.clear();
		fresults.open(filename.c_str(),ios_base::app);
		if(fresults.fail())
		{
			string erm="can not create output file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}


        double numchoicecases = 1+(caso.endchoice -caso.inichoice)/caso.choiceinc;


		//for(int choice=0;choice<numchoicecases;choice++)
		//	fresults<<"\t"<<missinfDD99[choice]<<"\t"<<missinfBD03a[choice]<<"\t"<<missinfBD03b[choice]<<"\t"<<missinfG07[choice]<<"\t"<<missinfD12[choice]<<"\t"<<missinfCR13[choice]<<"\t"<<caso.replicas<<endl;

        double ch = caso.inichoice;
		for(int choice=0;choice<numchoicecases;choice++){
            if(fabs(ch)<0.0000001) ch =0.0;
			fresults<<setprecision(1)<<ch<<"\t"<<setprecision(2)<<caso.s<<"\t"<<caso.Kmax<<"\t"<<setprecision(2)<<caso.minpheno<<"\t"<<caso.maxpheno<<"\t"<<(caso.maxpheno-caso.minpheno)<<setprecision(6)<<"\t"<<aveVarfem[choice]<<"\t"<<aveVarmale[choice]<<"\t"<<aveVarrest[choice]<<"\t"<<avecorrelDD99[choice]<<"\t"<<sdcorrelDD99[choice]<<"\t"<<missinfDD99[choice]<<"\t"<<avecorrelBD03a[choice]<<"\t"<<sdcorrelBD03a[choice]<<"\t"<<missinfBD03a[choice]<<"\t"<<avecorrelBD03b[choice]<<"\t"<<sdcorrelBD03b[choice]<<"\t"<<missinfBD03b[choice]<<"\t"<<avecorrelGauss[choice]<<"\t"<<sdcorrelGauss[choice]<<"\t"<<missinfGauss[choice]<<"\t"<<avecorrelD12[choice]<<"\t"<<sdcorrelD12[choice]<<"\t"<<missinfD12[choice]<<"\t"<<avecorrelCR13[choice]<<"\t"<<sdcorrelCR13[choice]<<"\t"<<missinfCR13[choice]<<"\t"<<avencorrelDD99[choice]<<"\t"<<sdncorrelDD99[choice]<<"\t"<<avencorrelBD03a[choice]<<"\t"<<sdncorrelBD03a[choice]<<"\t"<<avencorrelBD03b[choice]<<"\t"<<sdncorrelBD03b[choice]<<"\t"<<avencorrelGauss[choice]<<"\t"<<sdncorrelGauss[choice]<<"\t"<<avencorrelD12[choice]<<"\t"<<sdncorrelD12[choice]<<"\t"<<avencorrelCR13[choice]<<"\t"<<sdncorrelCR13[choice]<<"\t"<< aveIDD99[choice]<<"\t"<<  sdIDD99[choice]<<"\t"<< aveIBD03a[choice]<<"\t"<<  sdIBD03a[choice]<<"\t"<<  aveIBD03b[choice]<<"\t"<<  sdIBD03b[choice]<<"\t"<<  aveIGauss[choice]<<"\t"<< sdIGauss[choice]<<"\t"<< aveID12[choice]<<"\t"<< sdID12[choice]<<"\t"<< aveICR13[choice]<<"\t"<< sdICR13[choice]<<"\t"<<caso.replicas<<endl;

            ch+=caso.choiceinc;
		}

        fresults.close();
	}

    void writeCaso(const string& filename, struct casos& caso,const int index){

		ofstream fcaso; //ofstream will be opened for writing
		fcaso.clear();
		fcaso.open(filename.c_str(),ios_base::app);
		if(fcaso.fail())
		{
			string erm="can not create caso file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}


        double numchoicecases = 1+(caso.endchoice -caso.inichoice)/caso.choiceinc;

        double ch = caso.inichoice;

        string function;
        switch(index){
        case 0: function="DD99";break;
        case 1: function="BD03";break;
        case 2: function="G07b";break;
        case 3: function="Gauss";break;
        case 4: function="D12";break;
        case 5: function="FND";break;
        default:function="Wrong function identifier";
        }
        fcaso<<"Funcion: "<<function<<endl;
        fcaso<<"caso.inichoice"<<"\t"<<"numchoicecases"<<"\t"<<"caso.s"<<"\t"<<"caso.Kmax"<<"\t"<<setprecision(2)<<"caso.minpheno"<<"\t"<<"caso.maxpheno"<<"\t"<<"(caso.maxpheno-caso.minpheno)"<<"\t"<<"caso.replicas"<<endl;
        fcaso<<setprecision(1)<<caso.inichoice<<"\t"<<numchoicecases<<"\t"<<setprecision(2)<<caso.s<<"\t"<<caso.Kmax<<"\t"<<setprecision(2)<<caso.minpheno<<"\t"<<caso.maxpheno<<"\t"<<(caso.maxpheno-caso.minpheno)<<"\t"<<caso.replicas<<endl;

        fcaso.close();
	}

    void writephenotypes(const string& filename,const vector<int>& pops,const vector<double>& femphenotypes, const vector<double>& malephenotypes, const bool threecols){

        ofstream fphenotypes; //ofstream will be opened for writing
		fphenotypes.clear();
		fphenotypes.open(filename.c_str(),ios_base::app);
		if(fphenotypes.fail())
		{
			string erm="can not create phenotypes file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}

        if(threecols){
            if(malephenotypes.size()!=femphenotypes.size())
            {
                string erm="Distinct number of females "+NtoS(femphenotypes.size()) +" and males "+NtoS(malephenotypes.size())+ " in writephenotypes function\n";
                cout <<erm;
                CppTools::texterr("SpecTest",erm,true);
            }
            if(pops.size()!=femphenotypes.size())
            {
                string erm="Distinct number of females "+NtoS(femphenotypes.size()) +" and popIds "+NtoS(pops.size())+ " in writephenotypes function\n";
                cout <<erm;
                CppTools::texterr("SpecTest",erm,true);
            }
            fphenotypes<<"PopId\t"<<"Females"<<"\t"<<"Males"<<endl;
            for(int f=0;f<femphenotypes.size();f++)
                fphenotypes<<pops[f]<<"\t"<<femphenotypes[f]<<"\t"<<malephenotypes[f]<<endl;
            fphenotypes<<endl;
        }
        else{
            fphenotypes<<"Males"<<endl;
            for(int m=0;m<malephenotypes.size();m++)
                fphenotypes<<malephenotypes[m]<<endl;
            fphenotypes<<endl;
            fphenotypes<<"Females"<<endl;
            for(int f=0;f<femphenotypes.size();f++)
                fphenotypes<<femphenotypes[f]<<endl;
            fphenotypes<<endl;
        }

    }

    void writecorrelations(const string& filename,const vector<double>& correl, const vector<double>& tolerance,const vector<double>& scaledcorrel, const vector<double>& scaledtolerance, const double& maxrange, const vector<double>& dmax){

            ofstream fcorrels; //ofstream will be opened for writing
            fcorrels.clear();
            fcorrels.open(filename.c_str(),ios_base::app);
            if(fcorrels.fail())
            {
                string erm="can not create correlations file " +filename+"\n";
                cout <<erm;
                CppTools::texterr("SpecTest",erm,true);
            }



            fcorrels<<"WITHOUT SCALING\n"<<endl;
		    fcorrels<<"Correlation\tTolerance\tDMax\n"<<endl;
		    for(int n=0;n<tolerance.size();n++)
                fcorrels<<correl[n]<<"\t"<<tolerance[n]<<"\t"<<dmax[n]<<endl;

		    fcorrels<<endl;
		    fcorrels<<"\nSCALING:[0, "+NtoS(maxrange) +"]\n"<<endl;
		    fcorrels<<"Correlation\tTolerance\n"<<endl;

		    for(int n=0;n<scaledtolerance.size();n++)
                fcorrels<<scaledcorrel[n]<<"\t"<<scaledtolerance[n]<<endl;

		    fcorrels<<endl;

    }


    void writefunctionval(const string& filename, vector<vector<double> >& vals,vector<double>& males,vector<double>& females,struct casos& caso, const double& choiceval, const bool putcabecera){

        ofstream ffuncs; //ofstream will be opened for writing
		ffuncs.clear();
		ffuncs.open(filename.c_str(),ios_base::app);
		if(ffuncs.fail())
		{
			string erm="can not create functions output file " +filename+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
		}
        if(putcabecera){
            string cabecera="Choice	Va	Kmax	Num	female	male	D	DD99	BD03a	BD03b	Gauss	D12	CR13\n";
            ffuncs<<cabecera<<endl;
        }


		int nummatings = males.size();
		for(int n=0;n<nummatings;n++){
		    ffuncs<<setprecision(1)<<choiceval<<"\t"<<setprecision(2)<<caso.s<<"\t"<<caso.Kmax<<"\t";
            ffuncs<<n<<"\t"<<setprecision(6)<<females[n]<<"\t"<<males[n]<<"\t"<<fabs(females[n]-males[n]);
            for(int f=0;f<vals.size();f++)
                ffuncs<<"\t"<<vals[f][n];
            ffuncs<<endl;
		}

        ffuncs<<endl;

    }

	double combinations(const int& N, const int& K){

	// compute combinations C(N,K)
		if(K==0) return 1;
		if(K==1) return N;
		int dif = N-K;
		switch (dif){
			case 1: return N;
			case 2: return N*(N-1)/2;
		}

		double num,den,res;
		num = CppTools::factorial(N);
		den = CppTools::factorial(K)*CppTools::factorial(N-K);
		res=num/den;
		return res;
	}

    int readcorreldata(const string& filename,vector<int>& pops, vector<double>& x, vector<double>& y){
        string mensaje;
        int numpops=0;
        try{

            ifstream datafile; //ifstream el fichero se abrirá para lectura
            datafile.open(filename.c_str());

            if(datafile.fail())
                {mensaje = "can not find input file";throw mensaje;}

            string temp,word;
            int firstline=0;

            if(!pops.empty()) pops.clear();
            if(!x.empty()) x.clear();
            if(!y.empty()) y.clear();

        // ***** FIRST LINE  *****

                while(getline(datafile, temp)) // goes through file till eof
                {
                    if(datafile.fail())
                    { mensaje = "FORMAT ERROR-0 IN CORRELATIONS FILE";throw mensaje; }// exception
                    firstline++;
                    if(firstline==1){ //skip first line
                        continue;
                    } // close if firstline is 1
                    else{ // next line


                        // first word of each line to the vector of populations
                        numpops=atoi(CppTools::get_first_word(temp).c_str());
                        pops.push_back(numpops);
                        x.push_back(atof(CppTools::get_wordX(temp,2).c_str()));//ojo que wordX cuenta desde 1 no desde 0
                        y.push_back(atof(CppTools::get_wordX(temp,3).c_str()));
                    } // close else
                } // close while
            if(firstline==1) { mensaje = "FORMAT ERROR-2 IN CORRELATIONS FILE";throw mensaje; }// exception

            datafile.close();
            return numpops;
        } // close try
        catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
        return numpops;

    }

    vector<double> estimatematingvar(const vector<int>pops, vector<double>& vcorrel,const int numpops, vector<double>& females, vector<double>& males){


        double meanx,meany,sdx,sdy;
        int ini,end,popid;


//for(int f=(pops.size()-1);f>=0;f--)
  //  cout<<pops[f]<<"\t"<<females[f]<<"\t"<<males[f]<<endl;
//exit(1);
        if(numpops<=0){
            string erm="Numpops problem in estimateematingvar " +NtoS(numpops)+"\n";
			cout <<erm;
			CppTools::texterr("SpecTest",erm,true);
        }



        double r2,correl=0;
        vector<double> v;
        if(!vcorrel.empty()) vcorrel.clear();

        for(int i=0;i<(numpops);i++){

            popid=i+1;
            ini=CppTools::ufindindx(pops,popid);

            if(popid<numpops) end=CppTools::ufindindx(pops,popid+1);
            else end=pops.size();
//cout<<ini<<"\t"<<end<<endl;
            correl =computecorrel(males,females,ini,end,meanx,meany,sdx, sdy);

            vcorrel.push_back(correl);
            r2=correl*correl;

            v.push_back(sqrt(sdx*sdy)*pow((1.0-r2)/r2,0.25));


        }



        return v;
    }

}; // close class Experiment

	int main(int argc, char* argv[]){

 // Old arguments in the main function (from preference functions comparison program)
		string mensaje="Argument input problem";
		bool scaled=true; // if scaled is true, by default (see below) the interval will be [aveP, Dmax]
		bool proof=false;
		bool isoextreme=false;
		bool traitfixed=false;
		int numruns=1,KmaxMax=5;
		// this now will be the average and variance in the population Mp Vp
		double aveP=0.0,sdP=1.0; // mean and standard deviation of the trait at the metapopulation level
		double rangeini=aveP; // if scaling by default put the range to be rangeini the mean of the normal and the rangemax (Dmax) the standard deviation on the normal
		double rangeend=(sdP);
		double fixtraitval=-1.0;
  // 231013 New arguments in the main function for the microArea experiment
		// number of microareas
		int mAnum=50;
		//s^2
		double Vaini=0.01;
        fixtraitval=(rangeend-rangeini)/2.0;

		switch(argc){

            case 2:
                try{
                    numruns= (atoi(argv[1]));
                    break;
                }
                catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 3:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        rangeini=aveP; // if scaling by default put the range to be rangeini the mean of the normal and the rangeend (Dmax) the standard deviation on the normal
                        fixtraitval=(rangeend-rangeini)/2.0;

                        break;
                    }
                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 4:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        sdP=(atof(argv[3]));
                        rangeini=aveP; // if scaling by default put the range to be rangeini the mean of the normal and the rangeend (Dmax) the standard deviation on the normal
                        rangeend=(sdP);
                        fixtraitval=(rangeend-rangeini)/2.0;
                        break;
                    }
                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
             case 5:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        sdP=(atof(argv[3]));
                        isoextreme= (*argv[4]!='0');
                        rangeini=aveP; // if scaling by default put the range to be rangeini the mean of the normal and the rangemax (Dmax) the standard deviation on the normal
                        rangeend=(sdP);
                        fixtraitval=(rangeend-rangeini)/2.0;

                        break;
                    }
                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
			case 6:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        sdP=(atof(argv[3]));
                        isoextreme= (*argv[4]!='0');
                        KmaxMax=(atoi(argv[5]));

                        rangeini=aveP;
                        rangeend=(sdP);
                        fixtraitval=(rangeend-rangeini)/2.0;
                        break;
                    }
                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 7:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        sdP=(atof(argv[3]));
                        isoextreme= (*argv[4]!='0');
                        KmaxMax=(atoi(argv[5]));
                        scaled= (*argv[6]!='0');
                        rangeini=aveP;
                        rangeend=(sdP);
                        fixtraitval=(rangeend-rangeini)/2.0;
                        break;
                    }

                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 8:
                    try{
                        numruns= (atoi(argv[1]));
                        aveP=atof(argv[2]);
                        sdP=(atof(argv[3]));
                        isoextreme= (*argv[4]!='0');
                        KmaxMax=(atoi(argv[5]));
                        scaled= (*argv[6]!='0');
                        rangeini=atof(argv[7]);
                        rangeend=(sdP);
                        fixtraitval=(rangeend-rangeini)/2.0;
                        break;
                    }

                    catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 9:
                try{
                    numruns= (atoi(argv[1]));
                    aveP=atof(argv[2]);
                    sdP=(atof(argv[3]));
                    isoextreme= (*argv[4]!='0');
                    KmaxMax=(atoi(argv[5]));
                    scaled= (*argv[6]!='0');
                    rangeini=atof(argv[7]);
                    rangeend=atof(argv[8]);
                    fixtraitval=(rangeend-rangeini)/2.0;
                    break;
                }
                catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 10:
                try{
                    numruns= (atoi(argv[1]));
                    aveP=atof(argv[2]);
                    sdP=(atof(argv[3]));
                    isoextreme= (*argv[4]!='0');
                    KmaxMax=(atoi(argv[5]));
                    scaled= (*argv[6]!='0');
                    rangeini=atof(argv[7]);
                    rangeend=atof(argv[8]);
                    proof= (*argv[9]!='0');
                    fixtraitval=(rangeend-rangeini)/2.0;
                    break;
                }
                catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 11:
                try{
                    numruns= (atoi(argv[1]));
                    aveP=atof(argv[2]);
                    sdP=(atof(argv[3]));
                    isoextreme= (*argv[4]!='0');
                    KmaxMax=(atoi(argv[5]));
                    scaled= (*argv[6]!='0');
                    rangeini=atof(argv[7]);
                    rangeend=atof(argv[8]);
                    proof= (*argv[9]!='0');
                    traitfixed= (*argv[10]!='0');
                    fixtraitval=(rangeend-rangeini)/2.0;

                    break;
                }
                catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
            case 12:
                try{
                    numruns= (atoi(argv[1]));
                    aveP=atof(argv[2]);
                    sdP=(atof(argv[3]));
                    isoextreme= (*argv[4]!='0');
                    KmaxMax=(atoi(argv[5]));
                    scaled= (*argv[6]!='0');
                    rangeini=atof(argv[7]);
                    rangeend=atof(argv[8]);
                    proof= (*argv[9]!='0');
                    traitfixed= (*argv[10]!='0');
                    fixtraitval= atof(argv[11]);

                    break;
                }
                catch ( string){cout<<mensaje; CppTools::writeerr(mensaje); exit(1);}
		}

		if(numruns==0){

		    vector<int>pops;
		    vector<double> correl;
		    vector<double> v,males,females;
            bool scalingmetapop=false;
		    int numpops=Experiment().readcorreldata("datoscopulassaxatilis.txt",pops, females,males);

		    if(pops.size()!=males.size()){
                string erm="Dimension problem after reading correl data " +NtoS(pops.size())+"\t"+NtoS(males.size())+"\n";
                cout <<erm;
                CppTools::texterr("SpecTest",erm,true);
            }
            if(females.size()!=males.size()){
                string erm="Dimension problem 2 after reading correl data " +NtoS(females.size())+"\t"+NtoS(males.size())+"\n";
                cout <<erm;
                CppTools::texterr("SpecTest",erm,true);
            }

		    v =Experiment().estimatematingvar(pops,correl,numpops, females,males);
		    cout<<"WITHOUT SCALING\n"<<endl;
		    cout<<"Correlation\tTolerance\n"<<endl;
		    for(int n=0;n<v.size();n++)
                cout<<correl[n]<<"\t"<<v[n]<<endl;
            cout<<endl;

            // **** now perform SCALING from 0 to range end and compare

            vector<double> scaledmales, scaledfemales;
            vector<double>maxval;
            scaledmales=males;
            scaledfemales=females;
            // We want a different scaling for each population

            if(scalingmetapop){
                double maxrawel = Experiment().scalephenotypes(0,rangeend, scaledmales,scaledfemales);
                 // Maximum after traslation the min to 0
                cout<<maxrawel<<endl;
            }
            else{
                maxval=Experiment().scalephenotypeswithinpop(numpops, 0,rangeend, scaledmales,scaledfemales, pops);
            }
            string sfname="scaled_D"+NtoS(rangeend)+"_copulassaxatilis.xls";
            Experiment().writephenotypes(sfname,pops,scaledfemales,scaledmales,true); // boolean is for two cols (true) or one col (false)format

            vector<double> sv,scorrel;
            sv =Experiment().estimatematingvar(pops,scorrel,numpops, scaledfemales,scaledmales);
		    cout<<"\nSCALING:[0, "+NtoS(rangeend) +"]\n"<<endl;
		    cout<<"Correlation\tTolerance\n"<<endl;
		    for(int n=0;n<sv.size();n++)
                cout<<scorrel[n]<<"\t"<<sv[n]<<endl;

            Experiment().writecorrelations("estimtolerances.xls",correl, v,scorrel, sv, rangeend,maxval);
		}
		else{

			//Experiment experimento =Experiment();
			Experiment().justdoit(numruns,KmaxMax,aveP,sdP,rangeini,rangeend,scaled,proof,fixtraitval,traitfixed,isoextreme,mAnum,Vaini);
		}




		return 0;
	}



